import os

import dateparser

from legal_doc_processing import logger
from legal_doc_processing.utils import get_label_


def predict_violation_date(data: dict) -> list:

    # sent list
    sent_list = data.content.split("\n")
    date_ok = lambda j: ("19" or "20") in j
    date_sent_list = [(i, j) for i, j in enumerate(sent_list) if date_ok(j)]

    # find date ents
    date_list = list()
    for _, sent in date_sent_list:
        date_list.extend(get_label_(sent, "DATE", nlspa=data.nlspa))

    # if  not
    if not date_list:
        return [("", 1)]

    # parse date
    date_list = [dateparser.parse(i) for i in date_list if dateparser.parse(i)]

    # date sort and clean
    date = sorted(date_list)[0]
    date = str(date)[:10]

    return [(date, 1)]


from legal_doc_processing import logger


def predict_nature_de_sanction(data: dict) -> list:
    """ """

    sanctions = data._feature_dict["_extracted_sanctions"]

    sanct_selector = lambda i: (("monetar" and "penalt") or "pay" or "$") in i.lower()
    sanctions = [(i, j) for i, j in sanctions if not sanct_selector(i)]

    return sanctions



from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all

from legal_doc_processing.legal_doc.clean.defendant import (
    clean_ans,
    _sub_you_shall_not_pass,
)


def _question_helper(txt) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    k_list = [
        "accused",
        "defendant",
        "violate",
        "against",
        "filed",
        "judgement",
        "complaint",
    ]
    for kk in k_list:
        if kk in _txt:
            res.append(kk)

    return res


def _question_selector(key: str) -> list:
    """based on a key from _question helper find the list of good question to ask """

    qs = list()

    if "accuse" in key:
        qs.extend(
            [
                ("Who is the accused?", "who_accused"),
                ("Who are the accused?", "who_accuseds"),
            ]
        )
    if "defendant" in key:
        qs.extend(
            [
                ("Who is the defendant?", "who_defendant"),
                ("Who are the defendants?", "who_defendants"),
                # ("what are the defendants?", "what_defendant"),
                # ("what is the defendant?", "what_defendant"),
            ]
        )
    if "violat" in key:
        qs.extend(
            [
                ("Who is the violator?", "what_violator"),
                ("Who have made the violations?", "what_violator"),
                ("Who has made the violations?", "what_violator"),
                ("Who are the violators?", "what_violators"),
                # ("What is the violator?", "what_violator"),
                # ("What are the violators?", "what_violators"),
            ]
        )
    if "against" in key:
        qs.extend(
            [
                ("Who is the victim", "what_against"),
                ("Against who is the action?", "what_against"),
                ("Who is the action against to?", "what_against"),
                ("Against who?", "what_againsts"),
                # ("What is the violator?", "what_violator"),
                # ("What are the violators?", "what_violators"),
            ]
        )
    if "filed" in key:
        qs.extend(
            [
                ("Who recieve a complaint?", "what_violator"),
                ("Who is accused?", "what_violator"),
                ("Against who is the complaint?", "what_violator"),
                # ("Who are the violators?", "what_violators"),
                # ("What is the violator?", "what_violator"),
                # ("What are the violators?", "what_violators"),
            ]
        )
    if "judgement" in key:
        qs.extend(
            [
                ("Who is pursued?", "what_judgement"),
                ("Who is inculpated?", "what_judgement"),
                ("Who is under judgement?", "what_judgement"),
                ("Who is accused?", "what_judgement"),
                ("Against who is the judgement?", "what_judgement"),
                # ("What is the violator?", "what_violator"),
                # ("What are the violators?", "what_violators"),
            ]
        )
    if "complaint" in key:
        qs.extend(
            [
                ("Who is accused?", "what_judgement"),
                ("Who is inculpated?", "what_judgement"),
                ("Who is under judgement?", "what_judgement"),
                ("Who recieved a complaint ?", "what_judgement"),
                ("Against who is the complaint?", "what_judgement"),
            ]
        )
    return qs


def predict_defendant(obj: dict, threshold=0.2, n_sents: int = 5) -> list:
    """init a pipe if needed, then ask all questions and group all questions ans in a list sorted py accuracy """

    # pipe to avoid re init a pipe each time (+/- 15 -> 60 sec)
    nlpipe, nlspa = obj["nlpipe"], obj["nlspa"]

    # pers_org_all
    pers_org_all = obj["pers_org_all"] + _u(_sub_you_shall_not_pass(obj["pers_org_all"]))
    pers_org_all = _u(pers_org_all)

    # items
    h1, abstract = obj["h1"], obj["abstract"]
    abstract_sents = obj["abstract_sents"][:n_sents]
    ans = []

    # ask method
    for sent in abstract_sents:
        key_list = _question_helper(sent)
        for key in key_list:
            # from key to questions and from questions to answers
            quest_pairs = _u(_question_selector(key))
            ans.extend(ask_all(sent, quest_pairs, nlpipe=nlpipe))

    # clean ans
    cleaned_ans = clean_ans(ans)
    answer_label = "new_answer"
    if not len(cleaned_ans):
        cleaned_ans = [{answer_label: "--None--", "score": -1}]

    # merge ans
    merged_ans = merge_ans(cleaned_ans, label=answer_label)

    # filert by spacy entities
    consitant_ans = [i for i in merged_ans if i[answer_label] in pers_org_all]

    # filter by threshold
    flatten_ans = [(i[answer_label], i["cum_score"]) for i in consitant_ans]
    last_ans = [(i, j) for i, j in flatten_ans if j > threshold]

    return last_ans


if __name__ == "__main__":

    # import
    import time
    from legal_doc_processing.utils import get_pipeline, get_spacy
    from legal_doc_processing.legal_doc.utils import legal_doc_X_y
    from legal_doc_processing.legal_doc.legal_doc import LegalDoc

    # laod
    nlpipe = get_pipeline()
    nlspa = get_spacy()
    nlspa.add_pipe("sentencizer")

    # data
    threshold = 0.2
    n_sents = 7
    feature = "defendant"

    # structured_press_release_r
    df = legal_doc_X_y(features="defendant")
    df = df.iloc[:2, :]
    df["obj"] = [LegalDoc(i, nlpipe=nlpipe, nlspa=nlspa) for i in df.txt.values]

    # preds
    t = time.time()
    # 28 objects --> 181 secondes so --> +/-10 secondes per objects
    df["pred_" + feature] = df.obj.apply(lambda i: i.predict(feature))
    t = time.time() - t

    # 1st one
    one = df.iloc[0, :]
    one_txt = one.txt
    one_ob = self = one.obj
    obj = self.data_


from legal_doc_processing import logger


def predict_currency(obj: dict) -> list:
    """ """

    for k in ["€", "$", "£"]:
        if (k in obj["h1"]) or (k in obj["abstract"]):
            return [(k, 1)]

    return [(-1, -1)]



from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all

from legal_doc_processing.press_release.extracted_sanctions.questions import (
    _question_helper,
    _question_selector,
    _question_lister,
    _key_list,
)


def predict_extracted_sanctions(
    data: dict,
    h1_len_threshold: int = 15,
    content_n_sents_threshold: int = 6,
    threshold: float = 0.25,
) -> list:

    # sents
    h1 = [data.h1] if len(data.h1) > h1_len_threshold else [""]
    sent_list = h1 + data.content_sents[:content_n_sents_threshold]
    sent_list = [i.replace("\n", "").strip() for i in sent_list if i]

    filter_sents = lambda sent: any([(key.lower() in sent.lower()) for key in _key_list])
    sents_ok = [(i, j) for i, j in enumerate(sent_list) if filter_sents(j)]

    logger.info(f"sents_ok : {sents_ok} ")

    # quest
    ans_list = list()
    for i, sent in sents_ok:
        key_list = _question_helper(sent)
        if key_list:
            quest_pairs = _question_lister(key_list)
            ans_list.extend(ask_all(sent, quest_pairs, sent=sent, nlpipe=data.nlpipe))

    if not ans_list:
        return [("", 1)]

    # merged_ans
    answer_label = "answer"
    merged_ans = merge_ans(ans_list, label=answer_label)

    # filter by threshold
    flatten_ans = [(i[answer_label], i["cum_score"]) for i in merged_ans]
    last_ans = [(i, j) for i, j in flatten_ans if j > threshold]

    logger.info(f"last_ans : {last_ans} ")

    return last_ans


from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all, get_label_

from legal_doc_processing.press_release.court.questions import (
    _question_helper,
    _question_selector,
)

from legal_doc_processing.press_release.court.clean import final_clean


def predict_court(data: dict, threshold: float = 0.25) -> list:
    """ """

    # make sent list, and filter not cooperat in sent
    sent_list = data.content_sents
    court_ok = lambda j: ("court" or "tribunal" or "district" or "federal") in j
    court_sent_list = [(i, j) for i, j in enumerate(sent_list) if court_ok(j.lower())]

    # if no sents :
    if not len(court_sent_list):
        return [("", 1)]

    # questions
    ans_list = list()
    for i, sent in court_sent_list:
        quest_pairs = _u(_question_selector(_question_helper(sent)))
        ans_list.extend(ask_all(sent, quest_pairs, nlpipe=data.nlpipe))

    # logger.info(f"ans_list : {ans_list}")

    # threshold
    ans_list = _u([ans["answer"] for ans in ans_list if ans["score"] >= threshold])

    # if no ans :
    if not ans_list:
        return [("", 1)]

    # # filter org_list
    # org_list = get_label_(
    #     " ".join([j for _, j in court_sent_list]), "ORG", nlspa=data.nlspa
    # )
    # org_list = _u([i.lower().strip() for i in org_list])  # unique
    # is_in = lambda i: [(i.lower().strip() in org) for org in org_list]
    # ans = [i for i in ans if any(is_in(i))]

    # ans
    ans_list = [ans_list[0]]

    # clean
    ans_list = [final_clean(j) for j in ans_list]

    return [
        (j.title().strip().replace("U.s", "U.S").replace("Us", "U.S."), 1)
        for j in ans_list
    ]



from itertools import product

import pandas as pd

from legal_doc_processing import logger

from legal_doc_processing.utils import (
    _if_not_spacy,
    _if_not_pipe,
    get_label_,
    _ask,
    main_X_y,
)


def product_juridic_form():
    """ make a list of llc, LLC, LLC. etc etc """

    # cands
    cands = ["inc", "llc", "ltd", "corp"]

    # conatiner
    llc_list = list()

    # for each
    for i in cands:
        # various case
        sample = [str(i), str(i).lower(), str(i).capitalize(), str(i).upper()]
        # case * -- point ou pas -- * --  espace, ou pas --
        sample = product(sample, [", ", " ", ""], [".", ""])
        sample = [str(j + i + k) for i, j, k in sample]
        # extend
        llc_list.extend(sample)

    # sorted reverse lengt
    llc_list = [(len(i), i) for i in llc_list]
    llc_list = sorted(llc_list, reverse=True, key=lambda i: i[0])
    llc_list = [i[1] for i in llc_list]

    return llc_list


# def get_entities_pers_orgs(struct_doc: dict, n_paragraphs: int = 2, nlpspa=None) -> list:
#     """get entities PERSON and ORG from h1 and sub_article """

#     nlpspa = _if_not_spacy(nlpspa)

#     # sub article
#     sub_article = "\n".join(struct_doc["article"].split("\n")[:n_paragraphs])

#     # all pers all orgs from spacy entities
#     all_pers = get_pers(struct_doc["h1"], nlpspa) + get_pers(sub_article, nlpspa)
#     all_orgs = get_orgs(struct_doc["h1"], nlpspa) + get_orgs(sub_article, nlpspa)
#     pers_org_entities_list = all_pers + all_orgs

#     return pers_org_entities_list


def press_release_X_y(juridiction="", features="", sample=1.0):
    """ """

    assert juridiction in ["cftc", "cfbp", "doj", "sec", ""]

    # main
    main_df = main_X_y()

    # press_release_text not na
    main_df = main_df.loc[~main_df.press_release_text.isna(), :]

    # juridiction
    filter_jur = lambda i: juridiction.strip().lower() == str(i).strip().lower()
    if juridiction:
        jur_df = (
            main_df.loc[main_df["juridiction"].apply(filter_jur), :]
            if juridiction
            else main_df
        )
    else:
        jur_df = pd.DataFrame(main_df)

    # sample
    jur_df = jur_df.sample(frac=sample).reset_index(drop=True)

    # features
    if not features:
        drop_cols = [i for i in jur_df if (("document_" in i) or ("legal_doc" in i))]
        jur_df.drop(drop_cols, axis=1, inplace=True)
        return jur_df

    if isinstance(features, str):
        features = [features]
    keep_cols = [
        "folder",
        "press_release_link",
        "press_release_link_new",
        "press_release_text",
    ] + features

    features_df = jur_df.loc[:, keep_cols]

    return features_df



import os
import json
import logging
import requests

from logging import warning, info, debug, critical, error


class GeneralParams:
    """data storage object for general params (paths, logging etc etc) """

    # paths
    data = "data/"
    logs = "logs/"
    tmp = "tmp/"

    # logging
    logging_level = os.getenv("API_LOGLEVEL", "WARNING")
    log_in_file = False  # if True log in file else in stdout
    logging_filemode = "w"  # ['a', 'w'] if "w" delete old file else append
    logging_datefmt = "%Y-%m-%d %H:%M:%S"
    logging_format = "%(asctime)s.%(msecs)02d | %(levelname)-8s | %(message)s %(funcName)s | %(lineno)d | %(module)s | %(filename)s"


def setBasicConfig(filename: str, params, ext: str = ".log"):
    """Update logging.basicConfig from params.py

    args :
        filename (str): the filename of the root script
        ext (str) : the extension - optional - defaut is .log"""

    # clean filename
    if ".py" in filename:
        filename = filename.replace(".py", "")

    # base for basiConfig
    logging_dict = {
        "level": getattr(logging, params.logging_level),
        "format": params.logging_format,
        "datefmt": params.logging_datefmt,
    }

    # logfile
    # try:
    #     assert os.path.isdir(params.logs)
    # except Exception as e:
    #     os.mkdir(params.logs)
    logfile = f"{params.logs}{filename}{ext}"

    # if "w" as logging filemode rewrite logfile with header
    if (params.logging_filemode == "w") and params.log_in_file:
        open(logfile, "w").write("")

    # if log_in_file update basiConfig
    if params.log_in_file:
        logging_dict.update({"filename": logfile, "filemode": "a"})

    # basic config and logger
    logging.basicConfig(**logging_dict)


countries_list = [
    "Andorra",
    "United Arab Emirates",
    "Afghanistan",
    "Antigua and Barbuda",
    "Anguilla",
    "Albania",
    "Armenia",
    "Angola",
    "Antarctica",
    "Argentina",
    "American Samoa",
    "Austria",
    "Australia",
    "Aruba",
    "Aland Islands",
    "Azerbaijan",
    "Bosnia and Herzegovina",
    "Barbados",
    "Bangladesh",
    "Belgium",
    "Burkina Faso",
    "Bulgaria",
    "Bahrain",
    "Burundi",
    "Benin",
    "Saint Barthelemy",
    "Bermuda",
    "Brunei",
    "Bolivia",
    "Bonaire, Saint Eustatius and Saba ",
    "Brazil",
    "Bahamas",
    "Bhutan",
    "Bouvet Island",
    "Botswana",
    "Belarus",
    "Belize",
    "Canada",
    "Cocos Islands",
    "Democratic Republic of the Congo",
    "Central African Republic",
    "Republic of the Congo",
    "Switzerland",
    "Ivory Coast",
    "Cook Islands",
    "Chile",
    "Cameroon",
    "China",
    "Colombia",
    "Costa Rica",
    "Cuba",
    "Cabo Verde",
    "Curacao",
    "Christmas Island",
    "Cyprus",
    "Czechia",
    "Germany",
    "Djibouti",
    "Denmark",
    "Dominica",
    "Dominican Republic",
    "Algeria",
    "Ecuador",
    "Estonia",
    "Egypt",
    "Western Sahara",
    "Eritrea",
    "Spain",
    "Ethiopia",
    "Finland",
    "Fiji",
    "Falkland Islands",
    "Micronesia",
    "Faroe Islands",
    "France",
    "Gabon",
    "United Kingdom",
    "Grenada",
    "Georgia",
    "French Guiana",
    "Guernsey",
    "Ghana",
    "Gibraltar",
    "Greenland",
    "Gambia",
    "Guinea",
    "Guadeloupe",
    "Equatorial Guinea",
    "Greece",
    "South Georgia and the South Sandwich Islands",
    "Guatemala",
    "Guam",
    "Guinea-Bissau",
    "Guyana",
    "Hong Kong",
    "Heard Island and McDonald Islands",
    "Honduras",
    "Croatia",
    "Haiti",
    "Hungary",
    "Indonesia",
    "Ireland",
    "Israel",
    "Isle of Man",
    "India",
    "British Indian Ocean Territory",
    "Iraq",
    "Iran",
    "Iceland",
    "Italy",
    "Jersey",
    "Jamaica",
    "Jordan",
    "Japan",
    "Kenya",
    "Kyrgyzstan",
    "Cambodia",
    "Kiribati",
    "Comoros",
    "Saint Kitts and Nevis",
    "North Korea",
    "South Korea",
    "Kosovo",
    "Kuwait",
    "Cayman Islands",
    "Kazakhstan",
    "Laos",
    "Lebanon",
    "Saint Lucia",
    "Liechtenstein",
    "Sri Lanka",
    "Liberia",
    "Lesotho",
    "Lithuania",
    "Luxembourg",
    "Latvia",
    "Libya",
    "Morocco",
    "Monaco",
    "Moldova",
    "Montenegro",
    "Saint Martin",
    "Madagascar",
    "Marshall Islands",
    "North Macedonia",
    "Mali",
    "Myanmar",
    "Mongolia",
    "Macao",
    "Northern Mariana Islands",
    "Martinique",
    "Mauritania",
    "Montserrat",
    "Malta",
    "Mauritius",
    "Maldives",
    "Malawi",
    "Mexico",
    "Malaysia",
    "Mozambique",
    "Namibia",
    "New Caledonia",
    "Niger",
    "Norfolk Island",
    "Nigeria",
    "Nicaragua",
    "Netherlands",
    "Norway",
    "Nepal",
    "Nauru",
    "Niue",
    "New Zealand",
    "Oman",
    "Panama",
    "Peru",
    "French Polynesia",
    "Papua New Guinea",
    "Philippines",
    "Pakistan",
    "Poland",
    "Saint Pierre and Miquelon",
    "Pitcairn",
    "Puerto Rico",
    "Palestinian Territory",
    "Portugal",
    "Palau",
    "Paraguay",
    "Qatar",
    "Reunion",
    "Romania",
    "Serbia",
    "Russia",
    "Rwanda",
    "Saudi Arabia",
    "Solomon Islands",
    "Seychelles",
    "Sudan",
    "South Sudan",
    "Sweden",
    "Singapore",
    "Saint Helena",
    "Slovenia",
    "Svalbard and Jan Mayen",
    "Slovakia",
    "Sierra Leone",
    "San Marino",
    "Senegal",
    "Somalia",
    "Suriname",
    "Sao Tome and Principe",
    "El Salvador",
    "Sint Maarten",
    "Syria",
    "Eswatini",
    "Turks and Caicos Islands",
    "Chad",
    "French Southern Territories",
    "Togo",
    "Thailand",
    "Tajikistan",
    "Tokelau",
    "Timor Leste",
    "Turkmenistan",
    "Tunisia",
    "Tonga",
    "Turkey",
    "Trinidad and Tobago",
    "Tuvalu",
    "Taiwan",
    "Tanzania",
    "Ukraine",
    "Uganda",
    "United States Minor Outlying Islands",
    "United States",
    "Uruguay",
    "Uzbekistan",
    "Vatican",
    "Saint Vincent and the Grenadines",
    "Venezuela",
    "British Virgin Islands",
    "U.S. Virgin Islands",
    "Vietnam",
    "Vanuatu",
    "Wallis and Futuna",
    "Samoa",
    "Yemen",
    "Mayotte",
    "South Africa",
    "Zambia",
    "Zimbabwe",
    "Serbia and Montenegro",
    "Netherlands Antilles",
]



from legal_doc_processing import logger


def predict_country_of_violation(obj: dict) -> list:
    """ """

    auth = obj["feature_dict"]["extracted_authorities"].lower()

    for kk in ["sec", "doj", "cftc"]:
        if kk in auth:
            return [("United States of America", 1)]

    return [(-1, -1)]



from legal_doc_processing import logger


def _clean_defendants_1(txt: str) -> str:
    """ """
    if txt.lower() != "defendants":
        return ""

    if txt.lower() != "defendant":
        return ""

    return txt


def _clean_defendants_2(txt: str) -> str:
    """ """

    _txt = txt.lower()

    del_defendants = lambda i, defendant: i.strip().replace(defendant, "").strip()

    defendant_list = [
        "Defendants with",
        "Defendant with",
        "Defendants",
        "Defendant",
        "defendant",
        "defendants",
    ]

    for d in defendant_list:
        _txt = del_defendants(_txt, d.lower())

    return _txt


def _filter(txt: str) -> str:

    # list
    forbiden = ["cftc", "cftc complaint", "judgement"]

    # if == , then delete
    for f in txt:
        if txt.strip() == forbiden:
            return ""

    # else replace
    for f in forbiden:
        txt = txt.strip().replace(f, "").strip()

    return txt


def _clean_str_to_str(txt: str, defendants: bool) -> str:
    """ """

    txt = txt.strip()
    txt = txt.lower()
    # txt = _clean_defendants_1(txt)
    # txt = _clean_defendants_2(txt)
    # txt = _filter(txt)

    return txt


def _clean_list_to_list(ans_list: list, defendants=True) -> list:
    """ """

    ans_list = [
        _clean_str_to_str(i, defendants=defendants) for i in ans_list if i.lower()
    ]

    return ans_list


def clean_ans(ans):
    """ans is a list of dict. each dict is  : {answer:"foo", score:0.32}.
    for each dict,  add and _id and a new_ans based on the _you_shall_not_pass method
    the _you_shall_not_pass method is able to ditect:
     - completly inconsistant answer, if so the answer is droped
     - not so consistant answer, or non uniformized answer, if so the new_answer is the -more generic-
     version of ansxer"""

    # ans = copy.deepcopy(ans)

    # clean ans
    _ = [d.update({"_id": i}) for i, d in enumerate(ans)]
    _ = [d.update({"new_answer": _clean_list_to_list([d["answer"]])}) for d in ans]

    new_ans = list()
    for i, d in enumerate(ans):
        if len(d["new_answer"]) == 0:
            # ans.pop(i)
            pass
        if len(d["new_answer"]) == 1:
            # d["new_answer"] = list(d["new_answer"])[0]
            new_ans.append(
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": list(d["new_answer"])[0],
                }
            )
            # ans.pop(i)
        if len(d["new_answer"]) > 1:
            l = [
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": k,
                }
                for k in d["new_answer"]
            ]
            new_ans.extend(l)
            # ans.pop(i)

    return new_ans



def clean_ans(ans: str) -> str:
    """ """

    ans = ans.lower()

    clean_list = [
        "district judge",
        "federal judge",
        "judge",
        "general attorney",
        "attorney",
    ]

    # filter judge or attorney
    for item in clean_list:
        ans = ans.replace(item, "").strip()

    # capitalize
    ans = ans.title()

    # dummy \n
    ans = ans.replace(".\n", ". \n").replace("\n", " ").replace("  ", " ").strip()

    return ans


def clean_ans_list(ans_list: list) -> list:
    """ """

    _ = [d.update({"new_answer": clean_ans(d["answer"])}) for d in ans_list]

    return ans_list


from legal_doc_processing import logger
from legal_doc_processing._base import (
    Base,
)

from legal_doc_processing.legal_doc.utils import legal_doc_X_y as load_X_y


class LegalDoc(Base):
    """main LegalDoc class """

    def __init__(
        self,
        text: str,
        source: str = None,
        nlpipe=None,
        nlspa=None,
        n_lines: int = 6,
    ):

        Base.__init__(
            self,
            text=text,
            obj_name="LegalDoc",
            source=source,
            nlpipe=nlpipe,
            nlspa=nlspa,
        )

        # specs
        self.h1 = self.struct_text["header"]
        self.first_page = self.struct_text["pages"][0]
        if len(self.struct_text["pages"]) > 1:
            self.first_2_pages = (
                self.struct_text["pages"][0] + self.struct_text["pages"][1]
            )
        else:
            self.first_2_pages = self.struct_text["pages"][0]
        self.abstract = self.first_2_pages

        # set all
        self.set_all()


# def from_file(file_path, source, nlpipe=None, nlspa=None):
#     return base_from_file(file_path, source, LegalDoc, nlpipe=nlpipe, nlspa=nlspa)


# def from_text(txt, source, nlpipe=None, nlspa=None):
#     return base_from_text(txt, source, LegalDoc, nlpipe=nlpipe, nlspa=nlspa)


# def from_url(txt, source, nlpipe=None, nlspa=None):
#     return base_from_url(txt, source, LegalDoc, nlpipe=nlpipe, nlspa=nlspa)


class _LegalDoc:
    LegalDoc = LegalDoc
    load_X_y = load_X_y



import os

from legal_doc_processing import logger


def predict_folder(data: dict) -> list:
    """init a pipe if needed, then ask all questions and group all questions ans in a list sorted py accuracy """

    # _folder = obj["struct_text"]["folder"]
    # _folder = _folder.split(" ")[-1]
    # return [(_folder, 1)]

    return [("-- DUMMY --", 1)]



from google.cloud import storage


def get_client(file_path="source/amiable-dynamo-245709-9dcfa045897a.json"):
    """ """

    # client = storage.Client.from_service_account_json(file_path)
    return None


def get_buckets(client) -> list:
    """ """

    buckets = list(i for i in client.list_buckets())

    return buckets


def get_blobs(client, bucket="theolex_documents_processing") -> list:
    """ """

    # blobs = list(client.list_blobs(bucket))
    return None


def get_file_names(client, bucket="theolex_documents_processing") -> list:
    """ """

    blobs = list(client.list_blobs(bucket))
    files = [i.name for i in blobs]

    return files


# client = get_client()
client = None
# buckets = get_buckets(client)
# blobs = get_blobs(client)
# files = get_file_names(client)
files = None



from legal_doc_processing import logger

from legal_doc_processing.legal_doc.information_extraction.compliance_obligations import (
    predict_compliance_obligations,
)

from legal_doc_processing.legal_doc.information_extraction.court import (
    predict_court,
)

from legal_doc_processing.legal_doc.information_extraction.judge import (
    predict_judge,
)

from legal_doc_processing.legal_doc.information_extraction.cooperation_credit import (
    predict_cooperation_credit,
)

from legal_doc_processing.legal_doc.information_extraction.code_law_violation import (
    predict_code_law_violation,
)
from legal_doc_processing.legal_doc.information_extraction.country_of_violation import (
    predict_country_of_violation,
)
from legal_doc_processing.legal_doc.information_extraction.currency import (
    predict_currency,
)
from legal_doc_processing.legal_doc.information_extraction.decision_date import (
    predict_decision_date,
)
from legal_doc_processing.legal_doc.information_extraction.defendant import (
    predict_defendant,
)
from legal_doc_processing.legal_doc.information_extraction.extracted_authorities import (
    predict_extracted_authorities,
)

from legal_doc_processing.legal_doc.information_extraction.extracted_sanctions import (
    predict_extracted_sanctions,
)
from legal_doc_processing.legal_doc.information_extraction.extracted_violations import (
    predict_extracted_violations,
)
from legal_doc_processing.legal_doc.information_extraction.folder import predict_folder

# from legal_doc_processing.legal_doc.information_extraction.juridiction import (
#     predict_juridiction,
# )

from legal_doc_processing.legal_doc.information_extraction.justice_type import (
    predict_justice_type,
)

from legal_doc_processing.legal_doc.information_extraction.monitor import (
    predict_monitor,
)
from legal_doc_processing.legal_doc.information_extraction.monetary_sanction import (
    predict_monetary_sanction,
)

from legal_doc_processing.legal_doc.information_extraction.nature_de_sanction import (
    predict_nature_de_sanction,
)

from legal_doc_processing.legal_doc.information_extraction.nature_of_violations import (
    predict_nature_of_violations,
)
from legal_doc_processing.legal_doc.information_extraction.penalty_details import (
    predict_penalty_details,
)

# from legal_doc_processing.legal_doc.information_extraction.plaintiff import (
#     predict_plaintiff,
# )
from legal_doc_processing.legal_doc.information_extraction.reference import (
    predict_reference,
)

# from legal_doc_processing.legal_doc.information_extraction.sentence import (
#     predict_sentence,
# )
from legal_doc_processing.legal_doc.information_extraction.violation_date import (
    predict_violation_date,
)
from legal_doc_processing.legal_doc.information_extraction.type import (
    predict_type,
)



import re

from legal_doc_processing import logger


def predict_reference(obj: dict, length_treshold=50) -> list:
    """parse the first page line by line, matching a
    regex pattern refering to case feature
    example 'NO.: 14-CV-81216'
    return the result"""

    # use header and 1st page
    txt = obj["h1"] + obj["abstract"]
    first_page = txt.splitlines()
    first_page = [i for i in first_page if len(i) >= 4]

    # format result
    format_result = (
        lambda i: i.group(0)
        .upper()
        .replace("NO.", "")
        .replace("NO", "")
        .replace(":", "")
        .strip()
    )

    rr = None

    # first search smthg with - 99-CV-99999 -
    p = re.compile("\d*-?CV-\d+.*")
    for line in first_page:
        result = p.search(line.upper())
        if result:
            rr = format_result(result)
            break

    # first search smthg with - No.: -
    p = re.compile("NO[\.:]\s*.+")
    for line in first_page:
        result = p.search(line.upper())
        if result:
            rr = format_result(result)
            break

    if not rr:
        return [("--None--", -1)]

    rr = rr.split(",")[0].strip().replace("'", "")
    rr = rr.replace("—", "-")

    if "-" in rr:
        r_spilt = rr.split("-")
        rr = "-".join([i.strip() for i in r_spilt])

    return [(rr, 1)]


if __name__ == "__main__":

    # import
    import time
    from legal_doc_processing.utils import get_pipeline, get_spacy
    from legal_doc_processing.legal_doc.utils import legal_doc_X_y
    from legal_doc_processing.legal_doc.legal_doc import LegalDoc

    # laod
    nlpipe = get_pipeline()
    nlspa = get_spacy()
    nlspa.add_pipe("sentencizer")

    # data
    threshold = 0.2
    n_sents = 7
    feature = "reference"

    # structured_press_release_r
    df = legal_doc_X_y(features="defendant")
    df = df.iloc[:2, :]
    df["ld"] = [LegalDoc(i, nlpipe=nlpipe, nlspa=nlspa) for i in df.txt.values]

    # preds
    t = time.time()
    # 28 objects --> 181 secondes so --> +/-10 secondes per objects
    df["pred_" + feature] = df.ld.apply(lambda i: i.predict(feature))
    t = time.time() - t

    # 1st one
    one = df.iloc[0, :]
    one_txt = one.txt
    one_ld = self = one.ld
    obj = self.data


# from legal_doc_processing import logger

# from legal_doc_processing.decision.decision import Decision
# from legal_doc_processing.decision.decision import from_file, from_text
# from legal_doc_processing.decision.utils import (
#     decision_X_y as load_X_y,
# )



from legal_doc_processing import logger

import re


def clean_in_line_break(txt: str) -> str:
    """transform 'hello comment\nca va'  en 'hello comment ca va' """

    break_ = "--BREAK--"
    endline_ = "--ENDLINE--"

    lines = txt.splitlines()
    lines = [i.strip() for i in lines]
    txt_1 = "\n".join(lines)

    txt_2 = txt_1.replace(".\n", f".{endline_}\n")
    txt_3 = txt_2.replace("\n\n", f"\n{break_}\n")

    txt_4 = txt_3.replace("\n", " ")
    txt_5 = txt_4.replace(f" {break_} ", break_)
    txt_6 = txt_5.replace(f"{break_}", f"\n\n")
    txt_7 = txt_6.replace(f"{endline_}", f"\n")

    txt_8 = txt_7.replace("\n\n", "\n")

    return txt_8


def clean_very_short_lines(txt: str, threshold: int = 5) -> str:
    """clean lines too short """

    lines = txt.splitlines()
    lines = [i.strip() for i in lines if len(i.strip()) > threshold]
    new_txt = "\n".join(lines)

    return new_txt


def do_strip(txt: str) -> str:
    """ """

    lines = txt.splitlines()
    lines = [i.strip() for i in lines]
    new_txt = "\n".join(lines)

    return new_txt



class DataHandler:
    def __init__(self, obj):

        # pipe spacy
        self.nlpipe = obj.nlpipe
        self.nlspa = obj.nlspa
        # feature
        self._feature_dict = obj._feature_dict
        self.feature_dict = obj.feature_dict
        # text
        self.source = obj.source
        self.juridiction = obj.juridiction
        self.raw_text = obj.raw_text
        self.date = obj.date
        self.h1 = obj.h1
        self.content = obj.content
        self.abstract = obj.abstract
        # sents
        self.h1_sents = obj.h1_sents
        self.abstract_sents = obj.abstract_sents
        self.content_sents = obj.content_sents
        # entities
        self.pers_h1 = obj.pers_h1
        self.pers_abstract = obj.pers_abstract
        self.org_h1 = obj.org_h1
        self.org_abstract = obj.org_abstract
        self.date_h1 = obj.date_h1
        self.date_abstract = obj.date_abstract
        self.cost_h1 = obj.cost_h1
        self.cost_abstract = obj.cost_abstract
        self.pers_all = obj.pers_all
        self.org_all = obj.org_all
        self.pers_org_all = obj.pers_org_all
        self.date_all = obj.date_all
        self.cost_all = obj.cost_all


import requests


def get_text(url: str) -> str:
    """  """

    # print(url)
    if (not url) or (not isinstance(url, str)):
        # print("not_url")
        return ""
    try:
        res = requests.get(url)
        # print(res)
        txt = res.text
        print(txt[:30].replace("\n", "..."))
        return txt
    except Exception as e:
        # print(str(e) + "\n")
        return ""
    return ""


def add_base_url(
    url: str,
    URL_BASE: str = "https://storage.googleapis.com/theolex_documents_processing",
) -> str:
    try:
        if "https://" in url:
            return url
        url = url if url[0] == "/" else "/" + url
        return URL_BASE + url

    except Exception as e:
        # print(e)
        return ""
    return url


def replace_originial_to_text(url: str) -> str:
    """ """

    try:
        return url.replace("/original/", "/text/")
    except Exception as e:
        return ""


def replace_pdf_to_txt(url: str) -> str:
    """ """

    try:
        return url.replace(".pdf", ".txt")
    except Exception as e:
        return ""


def clean_url(url: str) -> str:
    """ """

    url = add_base_url(url)
    url = replace_originial_to_text(url)
    url = replace_pdf_to_txt(url)

    return url


# from legal_doc_processing.press_release.utils import product_juridiction_pairs


# def _filter_jur(token, cands: list = None):
#     """ """

#     if not cands:
#         cands = product_juridiction_pairs()

#     for k, v in cands.items():
#         if token.lower().strip() == k.lower().strip():
#             return v.upper()

#     return ""


# def _filter_the(txt: str) -> str:
#     """ """

#     if txt.lower().strip().startswith("the"):
#         txt = txt[3:].strip()

#     return txt


# def _sub_shall_not_pass(txt) -> str:
#     """ """

#     txt = _filter_the(txt)

#     txt = _filter_jur(txt)

#     if (len(txt) < 3) or (len(txt) > 24):
#         txt = ""

#     return txt


# def _you_shall_not_pass(txt: str) -> list:
#     """ """

#     txt = _sub_shall_not_pass(txt)

#     if not txt:
#         return []

#     return [txt]


# def clean_ans(ans):
#     """ """

#     # clean ans
#     _ = [d.update({"_id": i}) for i, d in enumerate(ans)]
#     _ = [d.update({"new_answer": _you_shall_not_pass(d["answer"])}) for d in ans]

#     new_ans = list()
#     for i, d in enumerate(ans):
#         if len(d["new_answer"]) == 0:
#             # ans.pop(i)
#             pass
#         if len(d["new_answer"]) == 1:
#             # d["new_answer"] = list(d["new_answer"])[0]
#             new_ans.append(
#                 {
#                     "_id": d["_id"],
#                     "question": d["question"],
#                     "start": d["start"],
#                     "end": d["end"],
#                     "score": d["score"],
#                     "answer": d["answer"],
#                     "new_answer": d["new_answer"][0],
#                 }
#             )
#             # ans.pop(i)
#         if len(d["new_answer"]) > 1:
#             l = [
#                 {
#                     "_id": d["_id"],
#                     "question": d["question"],
#                     "start": d["start"],
#                     "end": d["end"],
#                     "score": d["score"],
#                     "answer": d["answer"],
#                     "new_answer": k,
#                 }
#                 for k in d["new_answer"]
#             ]
#             new_ans.extend(l)
#             # ans.pop(i)

#     return new_ans



from legal_doc_processing import logger

from legal_doc_processing.press_release.compliance_obligations.clean import final_clean


def predict_compliance_obligations(data: dict) -> list:
    """ """

    # make sent list, and filter not compliant in sent
    sent_list = data.content_sents
    filter_compliant = lambda j: "complian" in j.lower()
    compl_sent_list = [j for j in sent_list if filter_compliant(j)]

    # if no sents :
    if not len(compl_sent_list):
        return [("", 1)]

    # clean
    compl_sent_list = [final_clean(j) for j in compl_sent_list]

    return [(j, 1) for j in compl_sent_list]


from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all

from legal_doc_processing.press_release.extracted_violations.clean import (
    _clean_str_to_str,
    _clean_list_to_list,
    clean_ans,
)

from legal_doc_processing.press_release.extracted_violations.questions import (
    _question_helper,
    _question_selector,
    _question_lister,
)


def predict_extracted_violations(
    data: dict,
    h1_len_threshold: int = 15,
    content_n_sents_threshold: int = 10,
    threshold: float = 0.25,
) -> list:
    """ """

    # sents
    h1 = [data.h1] if len(data.h1) > h1_len_threshold else [""]
    sent_list = h1 + data.content_sents[:content_n_sents_threshold]
    sent_list = [i.replace("\n", "").strip() for i in sent_list if i]

    # filter
    keys_list = [
        "accus",
        "charg",
        "violat",
        "judgement",
        "ordere",
        "settle",
        "impose",
        "pay",
        "suit",
        "allege",
    ]
    filter_sents = lambda sent: any([(key in sent.lower()) for key in keys_list])
    sents_ok = [(i, j) for i, j in enumerate(sent_list) if filter_sents(j)]

    # quest
    ans_list = list()
    for i, sent in sents_ok:
        key_list = _question_helper(sent)
        if key_list:
            quest_pairs = _question_lister(key_list)
            ans_list.extend(ask_all(sent, quest_pairs, sent=sent, nlpipe=data.nlpipe))

    if not ans_list:
        return [("", 1)]

    # merged_ans
    answer_label = "answer"
    merged_ans = merge_ans(ans_list, label=answer_label)

    # TODO
    # filter
    # delete no orgs or person in the result
    # delete all of the defendants form response

    # filter by threshold
    flatten_ans = [(i[answer_label], i["cum_score"]) for i in merged_ans]
    last_ans = [(i, j) for i, j in flatten_ans if j > threshold]

    return last_ans



from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all, cosine_similarity
from legal_doc_processing.press_release.defendant.questions import (
    _question_helper,
    _question_selector,
    _question_lister,
)
from legal_doc_processing.press_release.defendant.clean import (
    _sub_you_shall_not_pass,
    clean_ans,
)


def predict_defendant(
    data: dict,
    h1_len_threshold: int = 15,
    content_n_sents_threshold: int = 5,
    threshold: float = 0.25,
) -> list:
    """ """

    # sents
    h1 = [data.h1] if len(data.h1) > h1_len_threshold else [""]
    sent_list = h1 + data.content_sents[:content_n_sents_threshold]
    sent_list = [i.replace("\n", "") for i in sent_list if i]

    # quest
    ans_list = list()
    for sent in sent_list:
        key_list = _question_helper(sent)
        if key_list:
            quest_pairs = _question_lister(key_list)
            ans_list.extend(ask_all(sent, quest_pairs, sent=sent, nlpipe=data.nlpipe))

    if not ans_list:
        return [("", 1)]

    # clean ans
    cleaned_ans = clean_ans(ans_list)
    answer_label = "new_answer"
    if not len(cleaned_ans):
        return [("", 1)]

    # merge ans
    merged_ans = merge_ans(cleaned_ans, label=answer_label)

    # filert by spacy entities
    consitant_ans = [i for i in merged_ans if i[answer_label] in data.pers_org_all]

    # exclude judge
    judge_list = [
        i.lower().replace("judge", "").strip()
        for i in data.feature_dict["judge"].split(",")
    ]
    exclude_judge = lambda i: i[answer_label].strip().lower() not in judge_list
    if judge_list:
        consitant_ans = [i for i in consitant_ans if exclude_judge(i)]

    # filter by threshold
    flatten_ans = [(i[answer_label], i["cum_score"]) for i in consitant_ans]
    last_ans = [(i, j) for i, j in flatten_ans if j > threshold]

    return last_ans



from legal_doc_processing.utils import get_pipeline, get_spacy
from legal_doc_processing.press_release.press_release import PressRelease


def press_release() : 


    try : 
        with open("./data/boot/press_release.txt") as f : 
            return f.read()
        with open("./app/data/boot/press_release.txt") as f : 
            return f.read()
    except Exception as e : 
        txt =  """Release Number 7100-15

 

January 12, 2015

Federal Court in Florida Enters Order Freezing Assets in CFTC Foreign Currency Anti-
Fraud Action against Allied Markets LLC and its Principals Joshua Gilliland and Chawalit
Wongkhiao

CFIC Charges Jacksonville, Florida, Defendants with Operating a
Fraudulent Forex Pool and Misappropriating Customer Funds to Pay
Personal Expenses

Washington, DC — The U.S. Commodity Futures Trading Commission (CFTC) today announced that it filed a civil
enforcement Complaint in the U.S. District Court for the Middle District of Florida, charging Defendants Allied Markets
LLC, and its principals Joshua Gilliland and Chawalit Wongkhiao, all of Jacksonville, Florida, with operating a
fraudulent foreign currency (forex) commodity pool in violations of the Commodity Exchange Act (CEA) and CFTC
Regulations. In addition, none of the Defendants has ever been registered with the CFTC, as required.

On January 7, 2015, the day after the CFTC Complaint was filed under seal, U.S. District Judge Marcia Morales Howard
issued an emergency Order freezing and preserving assets under the Defendants’ control and prohibiting them from
destroying documents or denying CFTC staff access to their books and records. The Court has scheduled a hearing for
January 21, 2015, on the CFTC’s motion for a preliminary injunction.

The CFTC Complaint charges that, as early as January 2012, the Defendants fraudulently solicited more than $1 million
from members of the public to trade forex in a commodity pool. As alleged, Defendants Gilliland and Wongkhiao
misappropriated funds to pay their personal expenses. For example, instead of trading forex as promised, they allegedly
withdrew approximately $850,000 in pool participant funds from the Allied Markets forex account, spending more than
$64,000 of the stolen funds on restaurants and entertainment; approximately $33,000 on travel, hotels and rental cars;
and approximately $66,000 on rent for their residence in Jacksonville Beach, Florida. They also transferred at least
$83,200 to their personal bank accounts and withdrew approximately $139,000 in cash from automated teller machines
and at bank branches, the Complaint charges.

Defendants also used pool participant funds to pay purported trading profits and supposedly returned pool participants’
principal in the manner of a Ponzi scheme, according to the Complaint.

Specifically, according to the Complaint, the Defendants solicited members of the public to send them money for forex
trading by fraudulently guaranteeing specific trading returns and by making material misrepresentations regarding their
trading expertise and results, including that Defendants’ forex trading was generating large profits.

In its continuing litigation, the CFTC seeks full restitution to defrauded pool participants, disgorgement of any ill-gotten
gains, a civil monetary penalty, permanent registration and trading bans, and a permanent injunction against future
violations of the CEA and CFTC Regulations, as charged.

The CFTC appreciates the assistance of the Jacksonville Beach Police Department, the Jacksonville Sheriff's Office, the
Florida Office of Financial Regulation, the United States Marshals Service, and the National Futures Association.

CFTC Division of Enforcement staff members responsible for this action are Jonah McCarthy, Patricia Gomersall, Dmitriy
Vilenskiy, Jonathan Robell, John Einstman, and Paul Hayeck.

(See Complaint and Asset Freeze Order under Related Links.)
2K OK OOK OK OK KOK
CFTC’s Foreign Currency (Forex) Fraud and Commodity Pool Fraud Advisories

The CFTC has issued several customer protection Fraud Advisories that provide the warning signs of fraud, including the
Foreign Currency Trading (Forex) Fraud Advisory, which states that the CFTC has witnessed a sharp rise in Forex trading
scams in recent years and helps customers identify this potential fraud.

The CFTC has also issued a Commodity Pool Fraud Advisory, which warns customers about a type of fraud that involves
individuals and firms, often unregistered, offering investments in commodity pools.

Customers can report suspicious activities or information, such as possible violations of commodity trading laws, to the
CFTC Division of Enforcement via a Toll-Free Hotline 866-FON-CFTC (866-366-2382) or file a tip or complaint online.
Media Contact
Dennis Holden
202-418-5088

Last Updated: January 12, 2015"""

        return txt


def order() : 
    """ """

    try : 
        with open("./data/boot/order.txt") as f : 
            return f.read()
        with open("./app/data/boot/order.txt") as f : 
            return f.read()
    except Exception as e : 

        txt =   """UNITED STATES DISTRICT COURT
MIDDLE DISTRICT OF FLORIDA
Jacksonville Division

U.S. COMMODITY FUTURES
TRADING COMMISSION,
Plaintiff,
v. Case No. 3:15-cv-5-J-34MCR
ALLIED MARKETS LLC,
JOSHUA GILLILAND, and
CHAWALIT WONGKHIAOQO,

Defendants.

 

 

 

ORDER GRANTING PLAINTIFF’S EX PARTE
MOTION FOR STATUTORY RESTRAINING ORDER AND SCHEDULING
PRELIMINARY INJUNCTION HEARING

THIS CAUSE is before the Court on Plaintiff's Ex Parte Motion for Statutory
Restraining Order and Motion for Preliminary Injunction (Doc. S-4; Motion), filed on
January 5, 2015. Simultaneously with the Motion, the United States Commodity Futures
Trading Commission (Plaintiff or CFTC) filed a Complaint for Injunctive Relief, Civil
Monetary Penalty, and Other Equitable Relief (Doc. S-3) against Defendants Allied Markets
LLC, Joshua Gilliland, and Chawalit Wongkhiao (collectively, Defendants). In the Motion,
the CFTC moves, pursuant to Section 6c(a) of the Commodity Exchange Act (CEA), 7
U.S.C. § 13a-1(a) (2012), for an ex parte statutory restraining order freezing assets and
prohibiting the destruction of books, records, or other documents; and for an order requiring

Defendants to show cause why a preliminary injunction should not issue.
 

 

 

The Court has considered the pleadings, declarations, exhibits, and memorandum
filed in support of the CFTC’s Motion, and now, being fully advised, finds as follows:

l. The Court possesses jurisdiction over the parties and over the subject matter
of this action pursuant to Section 6c of the CEA, 7 U.S.C. § 13a-1 (2012).

om Venue properly lies in this District, pursuant to Section 6c(e) of the CEA,
7 U.S.C. § 13a-1(e) (2012).

as There is good cause to believe that Defendant Allied Markets LLC (“Allied
Markets”), by and through its agents, principals and control persons, Defendants Joshua
Gilliland and Chawalit Wongkhiao have engaged in, are engaging in, and may continue to
engage in violations of Sections 2(c)(2)(C)Gii)(D(cc), 4b(a)(2)(A) and (C), 4k(2), 4m(1), and
40(1) of the CEA, 7 U.S.C. §§ 2(c)(2)(C)Gu)(D(cc), 6b(a)(2)(A), (C), 6k(2), 6m(1), 60(1)
(2012); and CFTC Regulations 3.12, 4.20(a)-(c), 4.41(a), 5.2(b)(1) and (3), 5.3(a)(2)@), and
5.3(a)(2)(ii), 17 C.F.R. §§ 3.12, 4.20(a)-(c), 4.41 (a), 5.2(b)(1), (3), 5.3(a)(2)G), (41) (2014).

4, The CFTC has presented evidence that, since at least January 2012,
Defendants fraudulently solicited more than $1 million from members of the public to
participate in a supposed commodity pool purportedly trading leveraged or margined retail
off-exchange foreign currency contracts, commonly known as “forex.” The CFTC has also
presented evidence that Defendants misappropriated pool participants’ funds to pay
Defendants Gilliland’s and Wongkhiao’s personal expenses. Additionally, the CFTC has
presented evidence that Defendants failed to register with the CFTC as required, failed to

segregate customer funds, and failed to operate the commodity pool as a separate entity, and
 

 

failed to accurately report or disclose the performance of the relevant investments, as
required by the CEA and CFTC Regulations.

» There is good cause to believe that immediate and irreparable damage to the
Court’s ability to grant effective final relief in the form of monetary redress will occur from
the sale, transfer, assignment, or other disposition by Defendants of assets or records unless
Defendants are immediately restrained and enjoined by order of this Court;

6. There is good cause for entry of an order freezing assets owned, controlled,
managed, or held by or on behalf of, or for the benefit of, Defendants.

7: There is good cause for entry of an order prohibiting Defendants, their agents,
servants, employees, assigns, attorneys, and persons in active concert or participation with
Defendants, including any successor thereof, from destroying records and/or denying agents
of the CFTC immediate and complete access to Defendants’ books and records for inspection
and copying.

8. Absent the entry of this order, Defendants would have the ability to dissipate
or transfer assets and destroy books and records.

9. An ex parte statutory restraining order is warranted here to preserve the status
quo, protect customers from loss and damage, and enable the CFTC to fulfill its statutory
duties.

Accordingly, it is ORDERED:

10. ‘Plaintiff's Ex Parte Motion for Statutory Restraining Order and Motion for
Preliminary Injunction (Doc. S-4) is GRANTED, in part, and TAKEN UNDER

ADVISEMENT, in part.
A. To the extent the CFTC requests an ex parte statutory restraining order,
the Motion is GRANTED as set forth below.
B. To the extent the CFTC requests the entry of a preliminary injunction, the
Motion is TAKEN UNDER ADVISEMENT.
RELIEF GRANTED

I. ASSET! FREEZE

IT IS HEREBY ORDERED that:

11. Defendants? and their agents, servants, employees, assigns, attorneys,
including any successor thereof, and persons in active concert or participation with them,
who receive actual notice of this Order by personal service or otherwise, are immediately
restrained and enjoined from directly or indirectly transferring, selling, alienating,
liquidating, encumbering, pledging, leasing, loaning, assigning, concealing, dissipating,
converting, withdrawing, or otherwise disposing of any assets, wherever located, including

Defendants’ assets held outside the United States, except as otherwise ordered by the Court.

 

' For the purposes of this Order, the term “Assets” means any legal or equitable interest in, right to, or
claim to, any real or personal property, whether individually or jointly, directly or indirectly controlled, and
wherever located, including, but not limited to: chattels, goods, instruments, equipment, fixtures, general
intangibles, effects, leaseholds mail or other deliveries, inventory, checks, notes, accounts (including, but not
limited to, bank accounts and accounts at other financial institutions), credits, receivables, lines of credit,
contracts (including spot, futures, options, or swaps contracts), insurance policies, and all cash, wherever
located, whether within or outside the United States.

2 For purposes of this Order, the term “Defendants” refers to Allied Markets LLC, Joshua Gilliland,
and Chawalit Wongkhiao, and any person insofar as he or she is acting in the capacity of an officer, agent
servant, employee, or attorney of Defendants and any person who receives actual notice of this Order by
personal service or otherwise insofar as he or she is acting in concert or participation with Defendants.
“Defendants” also refers to any d/b/a, successor, affiliate, subsidiary, or other entity owned, controlled,
managed, or held by, on behalf of, or for the benefit of Allied Markets, Joshua Gilliland, or Chawalit
Wongkhiao.
 

12; This Order shall apply to any assets derived from or otherwise related to the
activities alleged in the CFTC’s complaint, regardless of when the asset is obtained.
However, the assets affected by this Order shall not include assets obtained after the effective
date of this Order, to the extent such assets are not derived from or otherwise related to the
activities alleged in the CFTC’s complaint.

Il. NOTICE TO FINANCIAL INSTUTIONS AND OTHERS

[3. Any financial or brokerage institution, business entity, or person that holds,
controls, or maintains custody of any account or asset titled in the name of, held for the
benefit of, or otherwise under the control of any Defendant, or has held, controlled, or
maintained custody of any such account or asset of any Defendant at any time since January
1, 2012 are hereby notified that this Order prohibits any Defendant and all other persons from
withdrawing, removing, assigning, transferring, pledging, encumbering, disbursing,
dissipating, converting, selling or otherwise disposing of Defendants’ assets, except as
directed by further order of the Court; provided, however, nothing in this Order shall limit the
discretion of any compliance official of any retail foreign exchange dealer or futures
commission merchant with which Defendants maintain an account to liquidate, or close out,
any and all open positions in Defendants’ account, in a prompt and orderly fashion in order
to avoid losses due to the asset freeze required by this Order.

Il. MAINTENANCE OF AND ACCESS TO BUSINESS RECORDS

IT IS FURTHER ORDERED that:
14. Defendants and all persons or entities who receive notice of this Order by

personal service or otherwise, are restrained from directly or indirectly destroying,
mutilating, erasing, altering, concealing or disposing of, in any manner, directly or indirectly,
any documents? that relate to the business practices or business or personal finances of any
Defendant.

IV. BOND NOT REQUIRED OF PLAINTIFF
IT IS FURTHER ORDERED that:

Ibs Because Plaintiff CFTC is an agency of the United States of America and has
made a proper showing under Section 6c(b) of the CEA, 7 U.S.C. § 13a-1(b) (2012), this
restraining order is granted without bond.

V. INSPECTION AND COPYING OF BOOKS AND RECORDS
IT IS FURTHER ORDERED that:

16. Defendants shall immediately allow representatives of the CFTC to inspect
the books, records, and other documents of Defendants and their agents including, but not
limited to, paper documents, electronically stored information, tape recordings, and computer
discs, wherever they may be situated and whether they are in the possession of Defendants or
others, and to copy said documents, data and records, either on or off the premises where
they may be situated.

VI. SERVICE OF ORDER AND ASSISTANCE OF UNITED STATES
MARSHALS SERVICE

IT IS FURTHER ORDERED that:
17. Copies of this Order may be served by any means, including facsimile

transmission, upon any financial institution or other entity or person that may have

 

3 For purposes of this Order, the term “document” is synonymous in meaning and equal in scope to the broad
usage of the term in Federal Rule of Civil Procedure 34(a).
 

possession, custody, or control of any documents or assets of any Defendant, or that may be
subject to any provision of this Order.

18. Representatives of the CFTC, the United States Marshals Service, the Florida
Office of Financial Regulation, and the police departments of Jacksonville and Jacksonville
Beach, Florida are specially appointed by the Court to effect service of this Order.

19, The United States Marshals Service, the Florida Office of Financial
Regulation, and the police departments of Jacksonville and Jacksonville Beach, Florida, in
order to ensure the safety of CFTC representatives, are authorized to accompany and assist
the CFTC in the service on Defendants of the summons, complaint, and supporting motions,
memoranda, and orders entered with this Order.

20. The CFTC shall immediately file proof of service with the Court once
accomplished.

VII. PRELIMINARY INJUNCTION HEARING

IT IS FURTHER ORDERED that:

21, This matter is set for a HEARING on Plaintiff's Motion for Preliminary
Injunction (Doc. S-4) on Wednesday, January 21, 2015, at 10:00 a.m., before the
undersigned at the Bryan Simpson United States Courthouse, 300 North Hogan Street,
Jacksonville, Florida, 32202, in Courtroom 10B. Counsel for Plaintiff and Defendants shall

appear in person and telephonic appearances will not be permitted.’

 

* All persons entering the Courthouse must present photo identification to Court Security Officers. Although
cell phones, laptop computers, and similar electronic devices generally are not permitted in the building,
attorneys may bring those items with them upon presentation to Court Security Officers of a Florida Bar card
(presentation of the Duval County Courthouse lawyer identification card will suffice) or Order of special
admission pro hac vice. However, all cell phones must be turned off while in the courtroom.
 

22. The following expedited briefing schedule shall govern this case:

A. Defendants shall have up to and including 5:00 p.m. on Thursday,

January 15, 2015, to file a memorandum in opposition to Plaintiffs Motion for

Preliminary Injunction (Doc. S-4), including any affidavits or declarations on which

they rely.

B. Plaintiff shall have up to and including 11:00 a.m. on Tuesday,

January 20, 2015, to file a reply, if mecessary, in support of its Motion for

Preliminary Injunction, which shall not exceed TEN (10) PAGES in length.

23, The hearing will be conducted in accordance with Local Rule 4.06, United
States District Court, Middle District of Florida, and Rule 65, Federal Rules of Civil
Procedure. The case does not appear to involve the exceptional situation wherein the Court
will allow the parties to submit evidence at the hearing. See Local Rule 4.06(b). Thus,
absent further order,’ the hearing will be limited to the written submissions and arguments of
counsel.

24. In issuing this Statutory Restraining Order, the Court understands that
Defendants have not yet been given an opportunity to be heard and emphasizes that it is not
making a final decision on any request for preliminary injunctive relief. However, the Court
is persuaded that issuing the Statutory Restraining Order until a full hearing can be held on

the CFTC’s request for preliminary injunctive relief is the lawful and proper action.

 

> McDonald’s Corp. v. Robertson, 147 F.3d 1301, 1312-13 (11th Cir. 1998) (“[W]here facts are bitterly
contested and credibility determinations must be made to decide whether injunctive relief should issue, an
evidentiary hearing must be held... . [W]here material facts are not in dispute, or where facts in dispute are not
material to the preliminary injunction sought, district courts need not hold an evidentiary hearing.”) In the
event either party believes such a hearing is warranted in this action, the party must, after conferring with
opposing counsel, file a motion requesting an evidentiary hearing and identifying the factual issues or dispute
which necessitate such relief,
 

Vill. FORCE AND EFFECT

IT IS FURTHER ORDERED that:
25. Unless earlier dissolved or extended by Order of the Court, the Statutory
Restraining Order granted herein shall expire at midnight on January 21, 2015.

IT IS SO ORDERED at Jacksonville, Florida on this 7th day of January, 2015, at

AO@.m.

aed

UNITED STATES DISTRICT JUDGE"""

        return txt

class Boot:
    """ """

    press_release = press_release()
    order = order()

    def boot() : 
        """boot module """ 

        nlpipe = get_pipeline()
        nlspa =  get_spacy()

        obj = PressRelease(text=press_release(), source="cftc",nlpipe=nlpipe, nlspa=nlspa)
        obj.predict_all()
        
        return obj



from legal_doc_processing.utils import uniquize as _u


def _question_helper(txt) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    cands = ["judge ", "attorney"]

    for cand in cands:
        if cand in _txt:
            res.append(cand)

    return res


def _question_selector(key: str) -> list:
    """based on a key from _question helper find the list of good question to ask """

    res = list()

    if "judge " in key:
        res.extend(
            [
                ("Who is the judge? ", "who_question"),
            ]
        )

    if "attorney" in key:
        res.extend(
            [
                ("Who is the attorney? ", "who_question"),
            ]
        )

    return res


def _question_lister(key_list: list) -> list:
    """from key_list return question list """

    question_list = []
    for key in key_list:
        question_list.extend(_question_selector(key))

    return _u(question_list)


import os
from pprint import pformat, pprint

from legal_doc_processing import logger

from legal_doc_processing.utils import get_spacy, get_pipeline, get_label_

from legal_doc_processing.press_release.utils import press_release_X_y

from legal_doc_processing.press_release.structure.utils import (
    clean_in_line_break,
    do_strip,
    clean_very_short_lines,
)


def give_doj_press_release_df():
    """ """

    df = press_release_X_y(juridiction="doj", sample=0.9)
    cols = ["folder", "press_release_text"]

    return df.loc[:, cols]


def give_doj_press_release_file():
    """ """

    fn = "./data/files/doj/airbus-se/press-release.txt"

    with open(fn, "r") as f:
        txt = f.read()

    # btxt = txt.encode("latin-1")
    # txt_decoded = btxt.decode("utf8")

    return txt


def first_clean(txt: str) -> str:
    """ """

    btxt = txt.encode("latin-1")
    txt_decoded = btxt.decode("utf8")

    # clean double breaks and fake lines
    new_txt_1 = clean_in_line_break(txt_decoded)

    # strip
    new_txt_2 = do_strip(new_txt_1)

    return new_txt_2


def split_intro_article(txt: str) -> str:
    """ """

    splitter = "for IMMEDIATE RELEASE"
    lines = txt.splitlines()

    # identifie and split
    idx_lines = [i for i, j in enumerate(lines) if splitter.lower() in j.lower()]
    # logger.info(idx_lines)
    assert len(idx_lines) > 0
    idx = idx_lines[0]
    intro_lines, article_lines = lines[: idx + 1], lines[idx + 1 :]

    # rejoin
    intro = "\n".join(intro_lines)
    article = "\n".join(article_lines)

    # find h1 in article
    article_lines = article.splitlines()

    first_true_sent_list = [
        i for i, j in enumerate(article_lines[:20]) if j.strip().endswith(".")
    ]
    first_true_sent_idx = first_true_sent_list[0]

    # swap h1 from article to intro
    intro_ok = intro + "\n" + "\n".join(article_lines[:first_true_sent_idx])
    article_ok = "\n".join(article_lines[first_true_sent_idx:])

    return intro_ok, article_ok


def extract_date(intro: str, nlspa) -> tuple:
    """ """

    date_list = get_label_(intro, label="DATE", nlspa=nlspa)

    if len(date_list) > 0:
        date = date_list[0]
    else:
        date = "-1"

    return date, intro


def extract_h1(intro: str, nlspa) -> tuple:
    """ """

    light_intro = clean_very_short_lines(intro)
    intro_lines = intro.splitlines()
    idx_list = [
        i for i, j in enumerate(intro_lines) if "IMMEDIATE RELEASE".lower() in j.lower()
    ]
    idx = idx_list[0]
    h1_lines = intro_lines[idx + 1 :]
    h1 = ". ".join(h1_lines).strip()
    h1 = h1 if h1[-1] == "." else h1 + "."

    return h1, intro


def structure_press_release(txt, nlspa=""):
    """ """

    # spacy
    if not nlspa:
        nlspa = get_spacy()
    try:
        nlspa.add_pipe("sentencizer")
    except Exception as e:
        pass

    dd = {
        "folder": "--ERROR--",
        "date": "--ERROR--",
        "h1": "--ERROR--",
        "article": "--ERROR--",
        "end": "--ERROR--",
        "error": "0",
    }

    try:

        # clean
        cleaned_txt = first_clean(txt)

        # intro article
        intro, article = split_intro_article(cleaned_txt)

        # date
        dd["date"], intro = extract_date(intro, nlspa)

        # h1
        dd["h1"], intro = extract_h1(intro, nlspa)

        # article
        dd["article"] = article

    except Exception as e:
        logger.error(e)
        dd["error"] = str(e)

    return dd



def _clean_on(txt: str) -> str:
    """ """

    _txt = txt.lower().strip()

    if _txt.startswith("on "):
        return _txt.replace("on ", "").strip().lower()

    return txt


def _clean_the_present(txt: str) -> str:
    """ """

    _txt = txt.lower().strip()
    if _txt == "present":
        return ""

    if "the present" in _txt:
        return ""

    return txt


def _strip_trailling_comma_dot(txt: str) -> str:
    """ """

    if not len(txt):
        return ""

    if txt[0] in [",", "."]:
        txt = txt[1:]

    if txt[-1] in [",", "."]:
        txt = txt[:-1]

    txt = txt.strip()

    return txt


# def avoid_dummy dates(date):
#     """avoid passing for a studid algo """

#     # validation funct
#     funct = lambda i: True if str(i) in date else False

#     # features to validate
#     features = [(range(1979, 2023), "years"), (range(1, 32), "day")]

#     # if a feature not in date retunr --None--
#     for feat, _ in features:
#         is_ok = bool(sum([funct(i) for i in feat]))
#         if not is_ok:
#             return "--None--"

#     return date


def _sub_shall_not_pass(txt: str) -> str:
    """ """

    txt = txt.lower().strip()
    if (len(txt) < 3) or (len(txt) > 35):
        return ""

    txt = _clean_on(txt)
    if (len(txt) < 3) or (len(txt) > 35):
        return ""

    txt = _clean_the_present(txt)
    if (len(txt) < 3) or (len(txt) > 35):
        return ""

    txt = _strip_trailling_comma_dot(txt)
    if (len(txt) < 3) or (len(txt) > 35):
        return ""

    return txt


def _you_shall_not_pass(txt: str) -> list:
    """ """

    txt = _sub_shall_not_pass(txt)

    if not txt:
        return []

    return [txt]


def clean_ans(ans: list) -> list:
    """ans is a list of dict. each dict is  : {answer:"foo", score:0.32}.
    for each dict,  add and _id and a new_ans based on the _you_shall_not_pass method
    the _you_shall_not_pass method is able to ditect:
     - completly inconsistant answer, if so the answer is droped
     - not so consistant answer, or non uniformized answer, if so the new_answer is the -more generic-
     version of ansxer "
     last but not least, and answer could be 'foo and bar' but this is indeed 2 answers
     'foo' and 'bar'. In this case we will create from one dict 2 dicts with same properties but separate
     new_ans
     before ans is a list of one dict -> [{answer:"foo and bar", score :0.123456},]
     after ans is a list of 2 dicts ->   [{new_answer:'foo', answer:"foo and bar", score :0.123456},
                                         {new_answer:'bar', answer:"foo and bar", score :0.123456}]"""

    # ans = copy.deepcopy(ans)

    # clean ans
    _ = [d.update({"_id": i}) for i, d in enumerate(ans)]
    _ = [d.update({"new_answer": _you_shall_not_pass(d["answer"])}) for d in ans]

    new_ans = list()
    for i, d in enumerate(ans):
        if len(d["new_answer"]) == 0:
            # ans.pop(i)
            pass
        elif len(d["new_answer"]) == 1:
            # d["new_answer"] = list(d["new_answer"])[0]
            new_ans.append(
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": d["new_answer"][0],
                }
            )
            # ans.pop(i)
        elif len(d["new_answer"]) > 1:
            l = [
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": k,
                }
                for k in d["new_answer"]
            ]
            new_ans.extend(l)
            # ans.pop(i)

    return new_ans



from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

# from legal_doc_processing.utils import merge_ans, ask_all

from legal_doc_processing.press_release.cooperation_credit.clean import final_clean


def predict_cooperation_credit(
    data: dict, threshold: float = 0.25, n_sents: int = 5
) -> list:
    """  """

    # make sent list, and filter not cooperat in sent :
    sent_list = data.content_sents
    filter_coop = lambda txt: "cooperat" in txt.lower()
    coop_sent_list = [i for i in sent_list if filter_coop(i)]

    # if no sents :
    if not len(coop_sent_list):
        return [("", 1)]

    # clean
    coop_sent_list = [final_clean(j) for j in coop_sent_list]

    return [(j, 1) for j in coop_sent_list]



def _question_helper(txt) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    cands = ["court", "district", "tribunal", "federal"]

    for cand in cands:
        if cand in _txt:
            res.append(cand)

    return res


def _question_selector(key: str) -> list:
    """based on a key from _question helper find the list of good question to ask """

    res = list()

    if "court" in key:  # court
        res.extend(
            [
                ("What is the court?", "who_court"),
            ]
        )
        return res

    if "district" in key:  # court
        res.extend(
            [
                ("What is the court?", "who_court"),
            ]
        )
        return res

    if "federal" in key:  # federal
        res.extend(
            [
                ("What is the court?", "who_court"),
            ]
        )
        return res

    if "tribunal" in key:  # tribunal
        res.extend(
            [
                ("What is the tribunal?", "who_court"),
            ]
        )

    return res


from legal_doc_processing import logger

# def _get_entities_money(obj: dict) -> list:
#     """get entities MONEY from h1 and sub_article """

#     nlspa = obj["nlspa"]

#     # sub article

#     # all pers all orgs from spacy entities
#     all_money = _u(obj["cost_all"])

#     # clean
#     all_init_money = _cast_as_int(all_money)
#     all_init_money = _u(all_init_money)

#     return all_init_money


def _cast_as_int(cleaned_ans):
    """transform a list of numbers in ints """

    MULTI = [("thousand", 1000), ("million", 1_000_000), ("billion", 1_000_000_000)]

    cleaned_ans = [i.lower().strip() for i in cleaned_ans]

    # delette € or $
    cleaned_ans = [
        i.replace("$", "").replace("€", "").replace("£", "") for i in cleaned_ans
    ]
    # logger.info(f"cleaned_ans: {cleaned_ans} ")

    # SPECIAL CASE :hundred of million
    spec_case = (
        lambda i: i.replace("hundred of", "100").strip()
        if "hundred of million" in i
        else i
    )
    cleaned_ans = [spec_case(i) for i in cleaned_ans]

    # SPECIAL CASE 100.000.000 usd
    spec_case = lambda i: i.replace(".", ",").strip() if (i.count(".") > 1) else i
    cleaned_ans = [spec_case(i) for i in cleaned_ans]

    # thousands as thousand
    cleaned_ans = [
        i.replace("thousands", "thousand")
        .replace("millions", "million")
        .replace("billions", "billion")
        .replace("hundreds", "hundred")
        for i in cleaned_ans
    ]
    # logger.info(f"cleaned_ans: {cleaned_ans} ")

    cleaned_ans_multi = list()
    for ans in cleaned_ans:
        multi = ""
        for k, _ in MULTI:
            if k in ans:
                multi = k
                break

        cleaned_ans_multi.append((ans, multi))

    # logger.info(f"cleaned_ans_multi: {cleaned_ans_multi} ")

    cleaned_ans_multi_2 = list()
    for numb, multi in cleaned_ans_multi:
        if not multi:
            # dump centimies
            numb = numb.split(".")[0]
            # easy, jsute keep the numbers
            numb = "".join([c for c in list(numb) if c.isnumeric()])
            numb = int(numb)
        else:
            # clean the numb: 1, 12 -> 1.22
            numb = numb.split(multi)[0].replace(",", ".").strip()

            # find last numberic and clean : a total of 3.12 -> 3.12
            try:
                cands_list = [i for i in numb.split(" ") if i[0].isnumeric()]
                cand = cands_list[-1].strip()
            except:
                raise AttributeError(f"{cands_list} ")

            # specific a 'total of for 3 000' ->  '3000'
            try:
                if cands_list[-2].strip()[0].isnumeric():
                    cand = str(cands_list[-2].strip()) + str(cand)
            except Exception as e:
                pass

            numb = float(cand.strip())

            # make 1.3 million -> 1.3 * 1 000 000 = 1 300 000
            for mm, k in MULTI:
                if mm == multi:
                    numb *= k

        cleaned_ans_multi_2.append(int(numb))

    logger.info(f"cleaned_ans_multi_2: {cleaned_ans_multi_2} ")

    return cleaned_ans_multi_2



juridiction_dict = {
    # cftc
    "the cftc": "cftc",
    "cftc": "cftc",
    "c.f.t.c": "cftc",
    "the commodity futures trading commission": "cftc",
    "commodity futures trading commission": "cftc",
    # doj
    "the doj": "doj",
    "doj": "doj",
    "d.o.j.": "doj",
    "the department of justice": "doj",
    "department of justice": "doj",
    # sec
    "the sec": "sec",
    "sec": "sec",
    "s.e.c.": "sec",
    "the securities and exchange commission": "sec",
    "securities and exchange commission": "sec",
    # cfpb
    "the cfbp": "cfbp",
    "the cfpb": "cfbp",
    "cfbp": "cfbp",
    "cfpb": "cfbp",
    "c.f.p.b": "cfbp",
    "the consumer financial protection bureau": "cfbp",
    "consumer financial protection bureau": "cfbp",
}


def product_juridiction_pairs():

    cftc_cands = [
        "the cftc",
        "cftc",
        "c.f.t.c",
        "the Commodity Futures Trading Commission",
        "Commodity Futures Trading Commission",
    ]
    doj_cands = [
        "the doj",
        "doj",
        "d.o.j.",
        "the department of justice",
        "department of justice",
    ]
    sec_cands = [
        "the sec",
        "sec",
        "s.e.c.",
        "the Securities and Exchange Commission",
        "Securities and Exchange Commission",
    ]

    cfbp_cands = [
        "the cfbp",
        "the cfpb",
        "cfbp",
        "cfpb",
        "c.f.p.b",
        "Consumer Financial Protection Bureau",
        "the Consumer Financial Protection Bureau",
    ]

    cands = dict()
    for k, _list in [
        ("cftc", cftc_cands),
        ("doj", doj_cands),
        ("sec", sec_cands),
        ("cfbp", cfbp_cands),
    ]:
        dd = {i.lower().strip(): k.lower().strip() for i in _list}
        cands.update(dd)

    return cands


from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all, cosine_similarity
from legal_doc_processing.press_release.type.questions import (
    _question_helper,
    _question_selector,
    _question_lister,
)


def predict_type(
    data: dict,
    h1_len_threshold: int = 15,
    content_n_sents_threshold: int = 5,
    threshold: float = 0.25,
) -> list:
    """ """

    # sents
    h1 = [data.h1] if len(data.h1) > h1_len_threshold else [""]
    sent_list = h1 + data.content_sents[:content_n_sents_threshold]
    sent_list = [i.replace("\n", "") for i in sent_list if i]

    type_list = [
        "enforcement",
        "compliance",
        "consent order",
        "judgement",
        "negociated ageerement",
        "settlement",
        "agreement",
        "engagement",
        "order",
        "complaint",
        "decision",
    ]

    # h1
    for h in h1:
        for type_cand in type_list:
            if type_cand in h.lower():
                return [(type_cand.title().strip(), 0.5)]

    # sent list
    for sent in sent_list:
        for type_cand in type_list:
            if type_cand in sent.lower():
                return [(type_cand.title().strip(), 0.5)]

    return [("", 1)]


import os

import requests

from legal_doc_processing import logger

from legal_doc_processing.utils import get_pipeline, get_spacy, get_label_, strize
from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.press_release.structure import structure_press_release
from legal_doc_processing.legal_doc.structure import structure_legal_doc

from legal_doc_processing.base.predict_handler import predict_handler
from legal_doc_processing.base.data_handler import DataHandler


class Base:
    """main Base class """

    def __init__(
        self,
        text: str,
        source: str,  # filename OR url with cftc doj ... in
        obj_name: str,  # PressRelesae or LegalDoc or Decision
        nlpipe=None,
        nlspa=None,
        abstract_n_lines=5,
    ):
        """ """

        # args check
        juridiction_list = ["cftc", "doj", "sec", "cfbp", "cfpb"]
        juridiction_cands = [i for i in juridiction_list if i in source]
        if not any(juridiction_cands):
            raise AttributeError(f"source arg must refers to one of {juridiction_list}")
        juridiction = juridiction_cands[0]
        obj_name_list = ["PressRelease", "Decision", "LegalDoc"]
        if not obj_name in obj_name_list:
            raise AttributeError(f"obj_name arg must refers to one of {obj_name_list}")

        # args as attr
        self.obj_name = obj_name
        self.juridiction = juridiction
        self.source = source

        # text
        self.raw_text = text
        self.date = ""
        self.h1 = ""
        self.header = ""
        self.content = ""
        self.struct_content = ""
        self.abstract = ""
        self.end = ""

        # pipe and spacy
        self.nlpipe = nlpipe if nlpipe else get_pipeline()
        self.nlspa = nlspa if nlspa else get_spacy()

        ######################

        # structure doc
        if obj_name == "PressRelease":

            struct_text = structure_press_release(
                text, juridiction=juridiction, nlspa=nlspa
            )
            self.date = struct_text["date"]
            self.h1 = struct_text["h1"]
            self.header = ""
            self.content = struct_text["article"]
            self.abstract = "\n".join(
                struct_text["article"].splitlines()[:abstract_n_lines]
            )

        elif obj_name == "LegalDoc":
            struct_text = structure_legal_doc(text, juridiction=juridiction, nlspa=nlspa)

        elif obj_name == "Decision":
            struct_text = {}

        ######################

        # prediction methods
        self._predict = predict_handler(obj_name)

        ######################

        # WARNING
        # the order of feature in feature list define the order of preidct methods called
        # this order is important
        # ie country of violations depedns of justice_type
        # penalty depends of violations

        # predict_country_of_violation NEED extracted_authorities
        # predict_extracted_authorities NEEDS judge
        # Justice_type NEEDS extracted_authorities

        self._feature_list = [
            "_cooperation_credit",
            "_court",
            "_currency",
            "_decision_date",
            "_extracted_sanctions",
            "_extracted_violations",  # ATTENTION PB extracted_violations vs _nature_of violations
            "_folder",
            "_judge",
            "_monitor",
            "_nature_de_sanction",  # _nature_de_sanction needs_extracted_sanctions
            "_nature_of_violations",  ###### ATTENTION PB extracted_violations vs _nature_of violations
            "_reference",
            "_extracted_authorities",  # depends of judge
            "_type",  # depends of _extracted_authorities
            "_justice_type",  # depends of _extracted_authorities
            "_defendant",  # depends of _judge and _extracted_authorities
            "_country_of_violation",  # depends of _extracted_authorities
            "_penalty_details",  # depends of _extracted_sanctions
            "_monetary_sanction",  # depends of _penalty_details
            "_compliance_obligations",  # depends of _extracted_sanctions
            # depends of predict authorities
            # "_violation_date",
        ]

        self.feature_list = [i[1:] for i in self._feature_list]
        _ = [setattr(self, k, [(None, -1)]) for k in self._feature_list]

    ######################

    @property
    def _feature_dict(self):
        return {k: getattr(self, k) for k in self._feature_list}

    @property
    def feature_dict(self):
        return {str(k[1:]): strize(getattr(self, k)) for k in self._feature_list}

    @property
    def data(self):
        return DataHandler(self)

    ######################

    def set_all(self):
        """ """

        # self._set_entities() / entities
        self.pers_h1 = _u(get_label_(self.h1, "PERSON", self.nlspa))
        self.pers_abstract = _u(get_label_(self.abstract, "PERSON", self.nlspa))
        self.org_h1 = _u(get_label_(self.h1, "ORG", self.nlspa))
        self.org_abstract = _u(get_label_(self.abstract, "ORG", self.nlspa))
        self.date_h1 = _u(get_label_(self.h1, "DATE", self.nlspa))
        self.date_abstract = _u(get_label_(self.abstract, "DATE", self.nlspa))
        self.cost_h1 = _u(get_label_(self.h1, "MONEY", self.nlspa))
        self.cost_abstract = _u(get_label_(self.abstract, "MONEY", self.nlspa))
        # all
        self.pers_all = _u(self.pers_h1 + self.pers_abstract)
        self.org_all = _u(self.org_h1 + self.org_abstract)
        self.pers_org_all = _u(self.pers_all + self.org_all)
        self.date_all = _u(self.date_h1 + self.date_abstract)
        self.cost_all = _u(self.cost_h1 + self.cost_abstract)

        # self.sents() / sents
        self.h1_sents = [i.text for i in self.nlspa(self.h1).sents if i.text.strip()]
        self.abstract_sents = [
            i.text for i in self.nlspa(self.abstract).sents if i.text.strip()
        ]
        self.content_sents = [
            i.text for i in self.nlspa(self.content).sents if i.text.strip()
        ]

    ######################

    @property
    def compliance_obligations(self):
        return strize(self._compliance_obligations)

    @property
    def cooperation_credit(self):
        return strize(self._cooperation_credit)

    @property
    def court(self):
        return strize(self._court)

    @property
    def country_of_violation(self):
        return strize(self._country_of_violation)

    @property
    def currency(self):
        return strize(self._currency)

    @property
    def decision_date(self):
        return strize(self._decision_date)

    @property
    def defendant(self):
        return strize(self._defendant)

    @property
    def extracted_authorities(self):
        return strize(self._extracted_authorities)

    @property
    def extracted_violations(self):
        return strize(self._extracted_violations)

    @property
    def folder(self):
        return strize(self._folder)

    @property
    def judge(self):
        return strize(self._judge)

    @property
    def justice_type(self):
        return strize(self._justice_type)

    @property
    def monetary_sanction(self):
        return strize(self._monetary_sanction)

    @property
    def monitor(self):
        return strize(self._monitor)

    @property
    def nature_de_sanction(self):
        return strize(self._nature_de_sanction)

    @property
    def nature_of_violations(self):
        return strize(self._nature_of_violations)

    @property
    def penalty_details(self):
        return strize(self._penalty_details)

    @property
    def reference(self):
        return strize(self._reference)

    @property
    def type(self):
        return strize(self._type)

    # @property
    # def violation_date(self):
    #     return strize(self._violation_date)

    ######################

    def predict(self, feature) -> str:
        """ """

        if feature == "all":
            return self.predict_all()

        # features who needs extracted_authorities
        if feature in [
            "country_of_violation",
            "type",
            "justice_type",
            "country_of_violation",
            "currency",
        ]:
            val = self._predict["extracted_authorities"](self.data)
            setattr(self, "_extracted_authorities", val)

        # Defendant need judge
        if feature == "defendant":
            val = self._predict["judge"](self.data)
            setattr(self, "_judge", val)

        # nature_de_sanction, penalty_details need extracted_sanctions
        if feature == "nature_de_sanction":
            val = self._predict["extracted_sanctions"](self.data)
            setattr(self, "_extracted_sanctions", val)

        if feature == "penalty_details":
            val = self._predict["extracted_sanctions"](self.data)
            setattr(self, "_extracted_sanctions", val)

        if feature in self.feature_list:
            try:
                val = self._predict[feature](self.data)
            except Exception as e:
                val = [("--Error-- " + str(e), 1)]
            setattr(self, "_" + feature, val)
            return val

        return AttributeError(f"Unknowned feature : {feature} ")

    def predict_all(self) -> str:
        """return self.predict("all") """

        for feature in self.feature_list:
            try:
                val = self._predict[feature](self.data)
            except Exception as e:
                val = [("--Error-- " + str(e), 1)]
            setattr(self, "_" + feature, val)
        return self.feature_dict

    ######################

    def __repr__(self):
        """__repr__ method """

        _pipe = "OK" if self.nlpipe else self.nlpipe
        _spa = "OK" if self.nlspa else self.nlspa
        _feat_dict = {k: v[:8] for k, v in self.feature_dict.items()}
        return f"{self.obj_name}(source:{self.source}, {_feat_dict}, pipe/spacy:{_pipe}/{_spa}"

    def __str__(self):
        """__str__ method """

        return self.__repr__()


# def base_from_text(text: str, source, Object, nlpipe=None, nlspa=None):
#     """ """

#     try:
#         return Object(text, source=source, nlpipe=nlpipe, nlspa=nlspa)
#     except Exception as e:
#         return e.__str__()


# def base_from_file(file_path: str, source, Object, nlpipe=None, nlspa=None):
#     """read a file and return  object """

#     try:
#         with open(source, "r") as f:
#             text = f.read()
#     except Exception as e:
#         return e.__str__()

#     return Object(text, source=source, nlpipe=nlpipe, nlspa=nlspa)


# def base_from_url(file_path: str, source, Object, nlpipe=None, nlspa=None):
#     """read a file and return  object """

#     text = requests.get(source)

#     return Object(text, source=source, nlpipe=nlpipe, nlspa=nlspa)



from legal_doc_processing.press_release.compliance_obligations.predict import (
    predict_compliance_obligations,
)

from legal_doc_processing.press_release.court.predict import (
    predict_court,
)

from legal_doc_processing.press_release.cooperation_credit.predict import (
    predict_cooperation_credit,
)
from legal_doc_processing.press_release.extracted_sanctions.predict import (
    predict_extracted_sanctions,
)

from legal_doc_processing.press_release.country_of_violation.predict import (
    predict_country_of_violation,
)
from legal_doc_processing.press_release.currency.predict import (
    predict_currency,
)

from legal_doc_processing.press_release.decision_date.predict import (
    predict_decision_date,
)

from legal_doc_processing.press_release.defendant.predict import (
    predict_defendant,
)

from legal_doc_processing.press_release.extracted_authorities.predict import (
    predict_extracted_authorities,
)

from legal_doc_processing.press_release.extracted_violations.predict import (
    predict_extracted_violations,
)

from legal_doc_processing.press_release.folder.predict import (
    predict_folder,
)

from legal_doc_processing.press_release.justice_type.predict import (
    predict_justice_type,
)

from legal_doc_processing.press_release.monitor.predict import (
    predict_monitor,
)
from legal_doc_processing.press_release.monetary_sanction.predict import (
    predict_monetary_sanction,
)

from legal_doc_processing.press_release.nature_de_sanction.predict import (
    predict_nature_de_sanction,
)

from legal_doc_processing.press_release.nature_of_violations.predict import (
    predict_nature_of_violations,
)
from legal_doc_processing.press_release.penalty_details.predict import (
    predict_penalty_details,
)

from legal_doc_processing.press_release.reference.predict import (
    predict_reference,
)

from legal_doc_processing.press_release.violation_date.predict import (
    predict_violation_date,
)
from legal_doc_processing.press_release.type.predict import (
    predict_type,
)
from legal_doc_processing.press_release.judge.predict import (
    predict_judge,
)



from tests import *

from legal_doc_processing.press_release.clean.monetary_sanction import (
    _cast_as_int,
)


class TestCastAsInt(unittest.TestCase):
    """ """

    def test_cast_as_int(self):
        def _test_cast_as_int():

            text_list = [
                ("hundred of million of dollars", 100_000_000),
                ("a total amount of 75 millions of dollars", 75_000_000),
                ("130 millions of dollars", 130_000_000),
                ("the fuckingsum of $3 000 thousand", 3_000_000),
                ("1.3 millions of dollars", 1_300_000),
                ("1,3 millions of dollars", 1_300_000),
                ("$1,000,000", 1_000_000),
                ("$1 000 000", 1_000_000),
                ("$1.000.000", 1_000_000),
                ("300000", 300_000),
                ("300 000", 300_000),
                ("300,000", 300_000),
                ("300.000", 300_000),
                ("300 thousands", 300_000),
                ("$3 thousands", 3_000),
                ("$1000", 1_000),
                ("$1 000", 1_000),
                ("$1,000", 1_000),
                ("$1thousand", 1_000),
            ]

            X, y = list(zip(*text_list))

            preds = _cast_as_int(X)
            ans = list(zip(preds, y))
            accuracy = sum([i == j for i, j in ans]) / len(ans)
            assert accuracy > 0.90

            return list(zip(X, y, preds))

        _test_cast_as_int()



import pandas as pd


def filter_juridiction(i: str, jur: str) -> str:
    """ """

    if not isinstance(i, str):
        return ""
    if not str("" + jur.lower() + "/") in i.lower():
        return ""

    return i


def filter_ext(i: str, ext: str) -> str:
    """ """

    ext = ext if ext[0] == "." else str("." + ext)
    if not isinstance(i, str):
        return ""
    if not ext.lower() in i.lower():
        return ""

    return i


def filter_press_release(i: str, press_release: bool) -> str:
    """ """

    press = "press-release"

    if press_release:
        return i if press in i else ""

    return "" if press in i else i


def get_document_link(doc_list):
    docs = [doc for doc in doc_list if "press-release" not in doc]
    accepted_words = ["deferred", "order", "complaint", "agreement", "indictment"]
    for word in accepted_words:
        for doc in doc_list:
            if word in doc:
                return doc

    return docs[0] if docs else None


def select_best_file(l: list) -> str:
    if not len(l):
        return ""
    if len(l) == 1:
        return l[0]

    return get_document_link(l)


def handle_multiple_file_problem(doc_pair_list: list) -> list:
    """ """

    # we are gonna, make a df
    tmp_list = [{"folder": i, "filename": j} for i, j in doc_pair_list]
    tmp_df = pd.DataFrame(tmp_list)
    tmp_df.head(10)

    # group by and render a list(tupple(str, list)) object
    tmp_clean_list = list()
    for k, sub_df in tmp_df.groupby("folder"):
        tu = (k, list(sub_df.filename.values))
        tmp_clean_list.append(tu)
    tmp_clean_list[:5]

    # clean
    tmp_clean_list = [(i, select_best_file(j)) for i, j in tmp_clean_list]
    print(tmp_clean_list[:5])

    return tmp_clean_list


def _question_helper(txt) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    cands = [
        # "impose",
        # "judgment",
        # "order",
        # "settl",
        # "defendant",
        # "charge",
        # "against",
        # "violate",
    ]

    for cand in cands:
        if cand in _txt:
            res.append(cand)

    return res


def _question_selector(key: str) -> list:
    """based on a key from _question helper find the list of good question to ask """

    res = list()

    if "impose" in key:  # impose
        res.extend(
            [
                ("Who is imposed?", "who_imposed"),
                ("Who are imposed?", "who_imposed"),
                # ("what are the victims?", "what_victims"),
                # ("what is the victim?", "what_victim"),
            ]
        )
    if "judgment" in key:  # judgment
        res.extend(
            [
                ("Who is under judgment?", "who_judgment"),
                ("Who are under judgment?", "who_judgment"),
                ("Who is convicted", "who_convicted"),
                ("Who are convicted?", "who_convicted"),
                # ("what are the victims?", "what_victims"),
                # ("what is the victim?", "what_victim"),
            ]
        )

    if "order" in key:  # order
        res.extend(
            [
                ("Who is ordered", "who_judgment"),
                ("Who are ordered?", "who_judgment"),
                ("Who recieve an order?", "who_order"),
                # ("what are the victims?", "what_victims"),
                # ("what is the victim?", "what_victim"),
            ]
        )

    if "settl" in key:  # settl
        res.extend(
            [
                ("Who recieve a settlement", "who_settled"),
                # ("Who are settled?", "who_settled"),
                # ("Who recieve an order?", "who_order"),
                # ("what are the victims?", "what_victims"),
                # ("what is the victim?", "what_victim"),
            ]
        )

    if "defendant" in key:  # defendant
        res.extend(
            [
                ("Who is the defendant?", "who_defendant"),
                ("Who are the defendants?", "who_defendants"),
                ("What is the defendant?", "what_defendant"),
                ("What are the defendants?", "what_defendants"),
            ]
        )

    if "charge" in key:  # charge
        res.extend(
            [
                ("Who is charged", "who_charged"),
                ("Who are charged?", "who_charged"),
                # ("what are the victims?", "what_victims"),
                # ("what is the victim?", "what_victim"),
            ]
        )

    if "against" in key:  # against
        res.extend(
            [
                ("Who is the victim?", "who_victim"),
                ("Who are the victims?", "who_victims"),
                # ("what are the victims?", "what_victims"),
                # ("what is the victim?", "what_victim"),
            ]
        )

    if "defendant" in key:  # defendant
        res.extend(
            [
                ("Who is the defendant?", "who_defendant"),
                ("Who are the defendants?", "who_defendants"),
                # ("what are the defendants?", "what_defendant"),
                # ("what is the defendant?", "what_defendant"),
            ]
        )

    if "violate" in key:  # violated
        res.extend(
            [
                #
                ("Who is the violator?", "who_violator"),
                ("Who are the violators?", "who_violators"),
                ("What is the violator?", "what_violator"),
                ("What are the violators?", "what_violators"),
                ("Who has violated?", "who_violated"),
                ("Who made the violations?", "who_violation"),
            ]
        )

    return res


from legal_doc_processing import logger


def _you_shall_not_pass(date: str) -> str:
    """avoid passing for a studid algo """

    # validation funct
    funct = lambda i: True if str(i) in date else False

    # features to validate
    features = [(range(1979, 2023), "years"), (range(1, 32), "day")]

    # if a feature not in date retunr --None--
    for feat, _ in features:
        is_ok = bool(sum([funct(i) for i in feat]))
        if not is_ok:
            return "--None--"

    return date


from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all

from legal_doc_processing.press_release.extracted_violations.clean import (
    _clean_str_to_str,
    _clean_list_to_list,
    clean_ans,
)

from legal_doc_processing.legal_doc.clean.defendant import (
    _sub_you_shall_not_pass,
)


def _question_helper(txt) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    # violated
    if "violat" in _txt.lower():
        res.append("violated")

    return res


def _question_selector(key: str):
    """based on a key from _question helper find the list of good question to ask """

    qs = list()

    # violated
    if "violat" in key:
        qs.extend(
            [
                ("what is the violations?", "what_violation"),
                ("what are the violations?", "what_violations"),
            ]
        )

    # elif "filed" in key:
    #     qs = [
    #         #
    #         ("Who has filed?", "who_filed"),
    #         ("Who filed?", "who_filed"),
    #     ]
    #     res.extend(qs)
    # else:
    #     qs = [
    #         #
    #         ("Who has reason?", "who_reason"),
    #         ("Who has filed?", "who_filed"),
    #     ]
    #     res.extend(qs)

    return qs


def predict_extracted_violations(obj: dict, threshold=0.2, n_sents: int = 12):
    """ """

    # pers_org_entities_list
    pers_org_all = obj["pers_org_all"] + _u(_you_shall_not_pass(obj["pers_org_all"]))
    pers_org_all = _u(pers_org_all)

    # items
    h1, abstract = obj["h1"], obj["abstract"]
    abstract_sents = obj["abstract_sents"][:n_sents]
    ans = []

    # ask medhod
    for sent in abstract_sents:
        key_list = _question_helper(sent)
        for key in key_list:
            # print(key)
            quest_pairs = _u(_question_selector(key))
            # print(quest_pairs)
            ans.extend(ask_all(sent, quest_pairs, nlpipe=obj["nlpipe"]))

    # clean ans
    cleaned_ans = ans
    answer_label = "answer"
    if not len(cleaned_ans):
        cleaned_ans = [{answer_label: "--None--", "score": -1}]

    # merge ans
    merged_ans = merge_ans(cleaned_ans, label=answer_label)

    # filert by spacy entities
    consitant_ans = [i for i in merged_ans if i[answer_label] not in pers_org_all]

    # filter by threshold
    flatten_ans = [(i[answer_label], i["cum_score"]) for i in consitant_ans]
    last_ans = [(i, j) for i, j in flatten_ans if j > threshold]

    return last_ans


if __name__ == "__main__":

    # import
    import time
    from legal_doc_processing.utils import get_pipeline, get_spacy
    from legal_doc_processing.legal_doc.utils import legal_doc_X_y
    from legal_doc_processing.legal_doc.legal_doc import LegalDoc

    # laod
    nlpipe = get_pipeline()
    nlspa = get_spacy()
    nlspa.add_pipe("sentencizer")

    # data
    threshold = 0.2
    n_sents = 7
    feature = "nature_of_violations"

    # structured_press_release_r
    df = legal_doc_X_y(features="defendant")
    df = df.iloc[:2, :]
    df["ld"] = [LegalDoc(i, nlpipe=nlpipe, nlspa=nlspa) for i in df.txt.values]

    # preds
    t = time.time()
    # 28 objects --> 181 secondes so --> +/-10 secondes per objects
    df["pred_" + feature] = df.ld.apply(lambda i: i.predict(feature))
    t = time.time() - t

    # 1st one
    one = df.iloc[0, :]
    one_txt = one.txt
    one_ld = self = one.ld
    obj = self.data


"""This module implements NLP methods to handle legal documentation. From text cleaning to information extraction, it centralizes main functionalities that simplify the work for text mining for this category of documents"""

import os
import logging

from params import GeneralParams as Params
from params import setBasicConfig

# pre set the logger
setBasicConfig(os.getenv("SERVICE_NAME", "LEGAL_DOC_PROCESSING"), Params)
logger = logging.getLogger()
logger.info("called")


# from legal_doc_processing._legal_doc import _LegalDoc as legal_doc
from legal_doc_processing.press_release.press_release import (
    _PressRelease as press_release,
)

from legal_doc_processing.downloader.downloader import Downloader as downloader
from legal_doc_processing.utils import Utils as utils
from legal_doc_processing.boot import Boot as boot

# from legal_doc_processing._decision import _Decision as decision



from legal_doc_processing import logger


def predict_currency(data: dict) -> list:
    """ """

    curr_dict = {"€": "EUR", "$": "USD", "£": "GBP"}

    for k in ["€", "$", "£"]:
        if (k in data.h1) or (k in data.abstract):
            return [(curr_dict[k], 1)]

    return [("USD", 0.8)]


from legal_doc_processing import logger


def predict_monitor(data: dict) -> list:
    """ """

    # make sent list, and filter not cooperat in sent
    sent_list = data.content_sents
    monitor_ok = lambda j: ("monitor" in j.lower())
    monitor_sent_list = [(i, j) for i, j in enumerate(sent_list) if monitor_ok(j)]

    # if no sents :
    if not len(monitor_sent_list):
        return [(0, 1)]

    for _, sent in monitor_sent_list:
        if ("indepen" and "complian" and "monitor") in sent.lower():
            return [(1, 1)]

    return [(0, 0.55)]


from legal_doc_processing.utils import uniquize as _u


def _question_helper(txt):
    """txt"""

    _txt = txt.lower()
    res = list()

    # involving

    if "involving" in _txt:
        res.append("involving")

    # failed
    if "failed to" in _txt:
        res.append("failed")

    # settle
    if "settle" in _txt:
        res.append("settle")

    # accused
    if ("accuse" in _txt) and ("for" in _txt):
        res.append("accused_for")
    if ("accuse" in _txt) and ("of" in _txt):
        res.append("accused_of")

    # charges
    if ("charg" in _txt) and ("for" in _txt):
        res.append("charged_for")
    if ("charg" in _txt) and ("with" in _txt):
        res.append("charged_with")

    # violated
    if ("violat" in _txt) and ("by" in _txt):
        res.append("violated_by")
    if ("allege" and "violate") in _txt:
        res.append("violated_by")

    # judge
    if ("judgment" in _txt) and ("for" in _txt):
        res.append("judgment_for")
    if ("judgment" in _txt) and ("in" in _txt):
        res.append("judgment_in")

    # order
    if ("order" in _txt) and ("from" in _txt):
        res.append("order_from")
    if ("order" in _txt) and ("for" in _txt):
        res.append("order_for")

    # impose
    if ("impose" in _txt) and ("for" in _txt):
        res.append("impose_for")

    # pay
    if ("pay" in _txt) and ("for" in _txt):
        res.append("pay_for")

    # allege
    if "allege" in _txt:
        res.append("allege")

    # suit
    if ("suit" and "against") in _txt:
        res.append("suit")

    return res


def _question_selector(key: str):
    """based on a key from _question helper find the list of good question to ask """

    qs = list()

    if "involving" in key:
        qs.extend(
            [
                ("What is involved?", "what_involved?"),
            ]
        )
    if "failed" in key:
        qs.extend(
            [
                ("What have they failed to do?", "what_failed"),
            ]
        )

    if "settle" in key:
        qs.extend(
            [
                ("What are the reason of the settlement?", "what_settlement"),
                # ("They are accused of what?", "accused_what"),
                # ("What is the reason of the accusation?", "what_accusation"),
            ]
        )
    if "allege" in key:  # accused
        qs.extend(
            [
                ("What are the allegations?", "what_acusation"),
                ("What is alleged?", "what_acusation"),
                # ("They are accused of what?", "accused_what"),
                # ("What is the reason of the accusation?", "what_accusation"),
            ]
        )

    if "accus" in key:  # accused
        qs.extend(
            [
                ("What is the accusation?", "what_acusation"),
                ("They are accused of what?", "accused_what"),
                ("What is the reason of the accusation?", "what_accusation"),
            ]
        )
    # charged
    elif "charge" in key:
        qs.extend(
            [
                ("What are the charges?", "what_charges"),
                ("For what are they charged?", "for_charged"),
                ("For what is he charged?", "for_charged"),
                ("For what is it charged?", "for_charged"),
            ]
        )

    if "suit" in key:
        qs.extend(
            [
                ("What are the violations?", "what_violations"),
                ("what are the reason of the persuit? ", "what_suit"),
            ]
        )

    if "violate" in key:  # violated
        qs.extend(
            [
                ("What are the violations?", "what_violations"),
                # ("They have violated what?", "violated_what"),
                # ("He has violated what?", "violated_what"),
                # ("It has violated what?", "violated_what"),
                ("What have they violated?", "what_violated"),
                ("What has he violated?", "what_violated"),
                ("What has it violated?", "what_violated"),
            ]
        )

    if "judge" in key:  # judgement
        qs.extend(
            [
                # ("What is the reason of the judgement?", "what_judge"),
                ("What are the reasons of the judgement?", "what_judge"),
                ("For what are they judged?", "for_what_judge"),
                ("For what are they under judgement?", "for_what_judge"),
                ("For what is he judge?", "for_what_judge"),
                # ("For what is it under judgement?", "for_what_judge"),
                # ("For what is he judge?", "for_what_judge"),
                # ("For what is it under judgement?", "for_what_judge"),
                # ("They are judged for what?", "for_what_judge"),
                # ("They are under judgedment for what?", "for_what_judge"),
                # ("He is judged for what?", "for_what_judge"),
                # ("He is under judgedment for what?", "for_what_judge"),
                # ("It is judged for what?", "for_what_judge"),
                # ("It is under judgedment for what?", "for_what_judge"),
            ]
        )

    if "order" in key:  # order
        qs.extend(
            [
                ("What is the reason of the order?", "what_judge"),
                ("What are the reasons of the order?", "what_judge"),
            ]
        )
    if "impose" in key:  # impose
        qs.extend(
            [
                ("What is the reason of the imposition?", "what_impose"),
                ("What are the reasons of the imposition?", "what_impose"),
                ("For what are they imposed?", "for_imposed"),
                ("For what is he imposed?", "for_imposed"),
                # ("For what is it imposed?", "for_imposed"),
            ]
        )
    if "pay" in key:  # pay
        qs.extend(
            [
                # ("What is the reason of the payement?", "what_payement"),
                # ("What are the reasons of the payement?", "what_payement"),
                ("What is the reason of the sanction ? ", "what_pay"),
                ("What are the violations? ", "what_pay")
                # ("What has he to pay for?", ""),
                # ("What have they to pay for?", ""),
            ]
        )

    if not qs:
        qs.extend(
            [
                ("What was the bad action?", "what_payement"),
                ("What is the bad action?", "what_payement"),
                ("What has he done wrong?", "what_payement"),
                ("What have they done wrong?", "what_payement"),
            ]
        )

    return qs


def _question_lister(key_list: list) -> list:
    """from key_list return question list """

    question_list = []
    for key in key_list:
        question_list.extend(_question_selector(key))

    return _u(question_list)


from legal_doc_processing import logger


def _clean_violation(txt: str) -> str:
    """ """

    viol_list = ["misuse", "misusing", "cftc", " cea ", "(cea)"]

    for viol in viol_list:
        if viol in txt:
            return ""

    return txt


def _force_at_least_2_tokens(txt: str) -> str:
    """ """

    if not " " in txt:
        return ""

    return txt


def _force_good_verbs(txt):
    """ """

    good_verbs = [
        "pay",
        "ceas",
        "ban",
        "freez",
        "preserv",
        "penalt",
        "disgord",
        "ban",
    ]

    one_of_them = [1 for i in good_verbs if i in txt]
    if not sum(one_of_them):
        return ""

    return txt


def _clean_str_to_str(txt: str) -> str:
    """ """

    txt = txt.lower().strip()
    txt = _clean_violation(txt)
    txt = _force_at_least_2_tokens(txt)
    txt = _force_good_verbs(txt)
    txt = txt.lower().strip()

    return txt


def _clean_list_to_list(ans_list: list, clean_str=_clean_str_to_str) -> list:
    """ """

    ans_list = [clean_str(i) for i in ans_list if i.strip()]

    return ans_list


def clean_ans(ans, clean_str=_clean_str_to_str, clean_list=_clean_list_to_list):
    """ """

    # ans = copy.deepcopy(ans)

    # clean ans
    _ = [d.update({"_id": i}) for i, d in enumerate(ans)]
    _ = [d.update({"new_answer": _clean_list_to_list([d["answer"]])}) for d in ans]

    new_ans = list()
    for i, d in enumerate(ans):
        if len(d["new_answer"]) == 0:
            # ans.pop(i)
            pass
        if len(d["new_answer"]) == 1:
            # d["new_answer"] = list(d["new_answer"])[0]
            new_ans.append(
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": list(d["new_answer"])[0],
                }
            )
            # ans.pop(i)
        if len(d["new_answer"]) > 1:
            l = [
                {
                    "_id": d["_id"],
                    "question": d["question"],
                    "start": d["start"],
                    "end": d["end"],
                    "score": d["score"],
                    "answer": d["answer"],
                    "new_answer": k,
                }
                for k in d["new_answer"]
            ]
            new_ans.extend(l)
            # ans.pop(i)

    return new_ans



import pdb

from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u
from legal_doc_processing.utils import get_label_

from legal_doc_processing.press_release.country_of_violation.countries_list import (
    countries_list,
)


def predict_country_of_violation(data: dict) -> list:
    """ """

    # authorities list
    juridiction = data.juridiction
    auth_list = data.feature_dict["extracted_authorities"].lower().split(";;")

    # cfbp and cftc -> USA
    for auth in ["cfbp", "cftc"]:
        if juridiction in auth:
            return [("United States", 1)]

    # make sent list, and filter not cooperat in sent
    sent_list = data.content_sents

    # find the list of countries
    countries_cands = list()
    for sent in sent_list:
        countries_cands.extend(get_label_(sent, "GPE", data.nlspa))
    countries_lowered = _u([i.lower().strip() for i in countries_cands])

    # filter
    _countries_list = [i.lower().strip() for i in countries_list]
    in_countries = lambda i: i.replace("the", "").strip() in _countries_list
    countries_filtered = [
        i.replace("the", "").strip() for i in countries_lowered if in_countries(i)
    ]

    # if not countries_filtered:
    #     pdb.set_trace()

    return [(i.title().strip(), 1) for i in countries_filtered]



from legal_doc_processing import logger

from legal_doc_processing.utils import get_spacy, get_pipeline, get_label_

from legal_doc_processing.press_release.structure.cftc import (
    structure_press_release as structure_cftc,
)
from legal_doc_processing.press_release.structure.doj import (
    structure_press_release as structure_doj,
)

from legal_doc_processing.press_release.structure.cfbp import (
    structure_press_release as structure_cfbp,
)

from legal_doc_processing.press_release.structure.sec import (
    structure_press_release as structure_sec,
)


def structure_press_release(txt, juridiction="", nlspa=None):
    """ """

    # spacy
    if not nlspa:
        nlspa = get_spacy()
    try:
        nlspa.add_pipe("sentencizer")
    except Exception as e:
        pass

    # if  juridiction --> EASY
    if juridiction == "cfbp":
        return structure_cfbp(txt, nlspa=nlspa)
    if juridiction == "cftc":
        return structure_cftc(txt, nlspa=nlspa)
    if juridiction == "doj":
        return structure_doj(txt, nlspa=nlspa)
    if juridiction == "sec":
        return structure_sec(txt, nlspa=nlspa)

    # ELSE TRY TO FIND JURIDICTION
    # identifiers
    cftc_pairs = (
        "cftc",
        ["CFTC", "Commodity Futures Trading Commission"],
    )

    doj_pairs = (
        "doj",
        ["DOJ", "Department of Justice"],
    )

    cfbp_pairs = (
        "cfbp",
        ["cfbp", "Consumer Financial Protection Bureau"],
    )

    sec_pairs = (
        "sec",
        ["Securities and Exchange Commission", "S.E.C", " SEC"],
    )

    # pairs and cands
    pairs = [cftc_pairs, doj_pairs, cfbp_pairs, sec_pairs]
    final_cands = list()

    # find a auth
    for auth, auth_cands in pairs:
        extract = [auth for cand in auth_cands if cand.lower() in txt.lower()]
        final_cands.extend(extract)

    assert len(final_cands) > 0
    auth = final_cands[0]

    # return funct
    if auth == "cfbp":
        return structure_cfbp(txt, nlspa=nlspa)
    if auth == "cftc":
        return structure_cftc(txt, nlspa=nlspa)
    if auth == "doj":
        return structure_doj(txt, nlspa=nlspa)
    if auth == "sec":
        return structure_sec(txt, nlspa=nlspa)



import os
from pprint import pformat, pprint

from legal_doc_processing import logger

from legal_doc_processing.utils import get_label_

from legal_doc_processing.press_release.structure.utils import (
    clean_in_line_break,
    do_strip,
    clean_very_short_lines,
)


def split_intro_article_1(txt: str, n=30) -> str:
    """ """

    txt_lines_30 = txt.splitlines()[:n]

    s0, s1, s2 = ("Washington D", "Washington, D", "Washington,D")
    is_in = lambda j: any([(j.startswith(s.lower())) for s in [s0, s1, s2]])
    idx_cands = [i for i, j in enumerate(txt_lines_30) if is_in(j.lower())]

    if not len(idx_cands):
        s = "washington"
        idx_cands = [i for i, j in enumerate(txt_lines_30) if j.lower().startswith(s)]

    idx = idx_cands[0]

    txt_lines = txt.splitlines()
    intro_lines, article_lines = txt_lines[:idx], txt_lines[idx:]

    intro, article = "\n".join(intro_lines), "\n".join(article_lines)

    return intro, article


def extract_id_1(intro: str) -> tuple:
    """ """

    # slplit
    intro_lines = intro.splitlines()
    # logger.info(intro_lines)
    intro_cleaned_lines = [i for i in intro_lines if i.strip()]
    # logger.info(intro_cleaned_lines)

    # find idx and _id
    cond_ok = lambda i: (
        ("198" in i) + ("199" in i) + ("200" in i) + ("201" in i) + ("202" in i)
    ) * ("-" in i)

    idx_cands = [i for i, j in enumerate(intro_cleaned_lines) if cond_ok(j)]
    # logger.info(idx_cands)

    idx = idx_cands[-1]
    _id = intro_cleaned_lines[idx]

    # clean intro
    idx_cands = [i for i, j in enumerate(intro_lines) if _id in j]
    idx = idx_cands[0]
    intro_lines[idx] = ""
    intro_ok = "\n".join(intro_lines)

    return _id, intro_ok


def extract_date_1(article: str, nlspa) -> tuple:
    """ """

    date_list = get_label_(article, label="DATE", nlspa=nlspa)
    if not len(date_list) > 0:
        return "-1", article

    return date_list[0], article


def extract_h1_1(intro: str) -> tuple:
    """ """

    light_intro = clean_very_short_lines(intro)
    light_intro.replace("\n\n", "\n").replace("\n\n", "\n")
    intro_lines = light_intro.splitlines()

    # clean
    del_list = ["Press Release", "immediate Release", "Home"]
    for item in del_list:
        intro_lines = [i for i in intro_lines if item.lower() not in i.lower()]

    h1 = " ".join(intro_lines).strip()
    h1 = h1 if h1[-1] == "." else h1 + "."

    if h1.startswith("r "):
        h1 = h1[2:]

    return h1, "-1"



from time import time

from legal_doc_processing import logger

from legal_doc_processing.utils import get_pipeline, get_spacy
from legal_doc_processing.base.base import Base
from legal_doc_processing.press_release.utils import press_release_X_y


class PressRelease(Base):
    """main press release doc class """

    def __init__(
        self,
        text: str,
        source: str,
        nlpipe=None,
        nlspa=None,
        n_lines: int = 6,
    ):

        Base.__init__(
            self,
            text=text,
            obj_name="PressRelease",
            source=source,
            nlpipe=nlpipe,
            nlspa=nlspa,
        )

        # set all
        self.set_all()


def press_release_df(
    juridiction="", nlspa=None, nlpipe=None, sample=0.25, max_init_time=3.0
):
    """ """

    assert juridiction in ["cftc", "cfbp", "doj", "sec", ""]

    max_init_time = 3.0

    # load
    if not nlpipe:
        nlpipe = get_pipeline()
    if not nlspa:
        nlspa = get_spacy()

    # dataframe
    df = press_release_X_y(juridiction=juridiction, sample=sample)

    # Press Releae

    juri = juridiction.lower().strip()
    if juridiction:
        # selec juridiction
        select_jur = lambda i: str(i).lower().strip() == juri
        df = df.loc[df.juridiction.apply(select_jur), :]
        # make pr
        make_pr = lambda i: PressRelease(i, source=juri, nlpipe=nlpipe, nlspa=nlspa)
        df["pr"] = df.press_release_text.apply(make_pr)
    else:
        # make pr
        make_pr = lambda i, j: PressRelease(i, source=j, nlpipe=nlpipe, nlspa=nlspa)
        df["pr"] = [make_pr(i, j) for i, j in zip(df.press_release_text, df.juridiction)]

    return df


# def from_file(file_path, source, nlpipe=None, nlspa=None):
#     return base_from_file(file_path, source, PressRelease, nlpipe=nlpipe, nlspa=nlspa)


# def from_text(txt, source, nlpipe=None, nlspa=None):
#     return base_from_text(txt, source, PressRelease, nlpipe=nlpipe, nlspa=nlspa)


# def from_url(txt, source, nlpipe=None, nlspa=None):
#     return base_from_url(txt, source, PressRelease, nlpipe=nlpipe, nlspa=nlspa)


class _PressRelease:
    PressRelease = PressRelease
    load_X_y = press_release_X_y
    load_df = press_release_df


def final_clean(txt: str) -> str:
    """ """

    clean = (
        lambda txt: txt.replace(".\n", ". \n")
        .replace("\n", " ")
        .replace("  ", " ")
        .replace("  ", " ")
    )

    return clean(txt)


from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u

from legal_doc_processing.utils import merge_ans, ask_all

from legal_doc_processing.legal_doc.clean.decision_date import (
    clean_ans,
    _sub_shall_not_pass,
)


def _question_helper(txt) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    kk_list = ["accused", "violate", "against", "filed", "judgment", "complaint"]

    for kk in kk_list:
        if kk in _txt:
            res.append(kk)

    return res


def _question_selector(key: str):
    """based on a key from _question helper find the list of good question to ask """

    qs = list()

    if "accuse" in key:
        qs.extend(
            [
                ("When the accusation has started?", "when_accused"),
                ("When strated the accusation?", "when_accuseds"),
                ("When strated the accusation?", "when_accuseds"),
                ("How long ago did the accusation start?", "when_accuseds"),
            ]
        )
    if "violate" in key:
        qs.extend(
            [
                ("When was the violations?", "when_violation"),
                ("When did the violations take place ?", "when_violation"),
                ("When did the violations take place ?", "when_violations"),
            ]
        )
    if "filed" in key:
        qs.extend(
            [
                ("When was filed a complaint?", "when_violation"),
                ("When was the complaint filed?", "when_violation"),
                ("When were the complaints filed?", "when_violations"),
            ]
        )
    if "judgement" in key:
        qs.extend(
            [
                ("When has begun the judgement?", "when_judgement"),
                ("When started the judgement?", "when_judgement"),
                ("When was decided the judgement?", "when_judgement"),
                ("When was filed the judgement?", "when_judgement"),
            ]
        )
    if "complaint" in key:
        qs.extend(
            [
                ("When has begun the complaint?", "when_complaint"),
                ("When started the complaint?", "when_complaint"),
                ("When was decided the complaint?", "when_complaint"),
                ("When was filed the  complaint?", "when_complaint"),
            ]
        )

    return qs


def predict_decision_date(obj: dict, threshold=0.2, n_sents: int = 6) -> list:

    # pipe to avoid re init a pipe each time (+/- 15 -> 60 sec)
    # win lots of time if the method is used in a loop with 100 predictions
    nlpipe, nlspa = obj["nlpipe"], obj["nlspa"]

    # date_all
    # we will use this one later to make a filter at the end
    date_all = obj["date_all"] + [_sub_shall_not_pass(i) for i in obj["date_all"]]
    date_all = _u(date_all)

    # items
    # we will work on h1 and / or article but just 2 or 3 1st paragraphs
    h1, abstract = obj["h1"], obj["abstract"]
    abstract_sents = obj["abstract_sents"][:n_sents]
    ans = []

    # ask method
    # for each sentence
    for sent in abstract_sents:
        # print(sent)
        # key list
        key_list = _question_helper(sent)
        for key in key_list:
            # print(key)
            # from key to questions and from questions to answers
            quest_pairs = _u(_question_selector(key))
            ans.extend(ask_all(sent, quest_pairs, nlpipe=nlpipe))

    # clean ans
    # ans is a list of dict, each dict has keys such as answer, score etc
    # for each answer we will clean this answer and create a new_answer more accurate
    cleaned_ans = clean_ans(ans)
    answer_label = "new_answer"
    if not len(cleaned_ans):
        cleaned_ans = [{answer_label: "--None--", "score": -1}]

    # merge ans
    # based on new_answer we will make a groupby adding the scores for each new ans in a cumulative score
    # example [{new_ans : hello, score:0.3},{new_ans : hello, score:0.3}, ]
    # will become  [{new_ans : hello, score:0.6},]
    merged_ans = merge_ans(cleaned_ans, label=answer_label)

    # filert by spacy entities
    # we are sure that a personn or an org is NOT a violations so
    # if a prediction is in pers_org_entities_list, plz drop it
    consitant_ans = [i for i in merged_ans if i[answer_label] in date_all]

    # filter by threshold
    # we need to filter the score above which we consider that no a signe score but a
    # cumulative score (much more strong, accurante and solid) will be droped
    flatten_ans = [(i[answer_label], i["cum_score"]) for i in consitant_ans]
    last_ans = [(i, j) for i, j in flatten_ans if j > threshold]

    return last_ans


if __name__ == "__main__":
    # import
    import time
    from legal_doc_processing.utils import get_pipeline, get_spacy
    from legal_doc_processing.legal_doc.utils import legal_doc_X_y
    from legal_doc_processing.legal_doc.legal_doc import LegalDoc

    # laod
    nlpipe = get_pipeline()
    nlspa = get_spacy()
    nlspa.add_pipe("sentencizer")

    # data
    threshold = 0.2
    n_sents = 7
    feature = "decision_date"

    # structured_press_release_r
    df = legal_doc_X_y(features="defendant")
    df = df.iloc[:2, :]
    df["obj"] = [LegalDoc(i, nlpipe=nlpipe, nlspa=nlspa) for i in df.txt.values]

    # preds
    t = time.time()
    # 28 objects --> 181 secondes so --> +/-10 secondes per objects
    df["pred_" + feature] = df.obj.apply(lambda i: i.predict(feature))
    t = time.time() - t

    # 1st one
    one = df.iloc[0, :]
    one_txt = one.txt
    one_ob = self = one.obj
    obj = self.data_


from legal_doc_processing import logger


def predict_nature_of_violations(data: dict) -> list:
    """ """

    violations = data.feature_dict["extracted_violations"].lower()

    natures = list()
    if "money laundering" in violations.lower():
        natures.append("Money Laundering")

    if ("foreign corrupt practices act" or "fcpa") in violations:
        natures.append("FCPA")

    if "Foreign Bribery".lower() in violations:
        natures.append("Foreign Bribery")

    if "Homeland Security".lower() in violations:
        natures.append("Homeland Security")

    if "spoofing".lower() in violations:
        natures.append("Spoofing")

    if "Fraud Scheme".lower() in violations:
        natures.append("Fraud Scheme")

    return [(i, 1) for i in natures]



import os
import pdb
from subprocess import call

import numpy as np
import pandas as pd

from sklearn.feature_extraction.text import TfidfVectorizer

import spacy
from transformers import pipeline


from legal_doc_processing import logger


def dummy_accuracy(y, pred) -> int:
    """ """

    try:
        y, pred = int(y), int(pred)
        val = int(y == pred)
        logger.info(f"val {val} ")
        return val

    except Exception as e:
        logger.info(f"e {e} ")
        return -1

    return -2


def sub_cosine_similarity(list_corpus: list) -> object:
    """ """

    vect = TfidfVectorizer(min_df=1, stop_words="english")

    tfidf = vect.fit_transform(list_corpus)
    pairwise_similarity = tfidf * tfidf.T

    return pairwise_similarity


def cosine_similarity(y: str, pred: str) -> float:
    """eval accuracy based on cosine similarity of 2 list of answers """

    # check if args are OK

    # separer y et pred (string avec virgugles) en liste de string
    y_list = [i.strip().lower() for i in y.split(",")]
    pred_list = [i.strip().lower() for i in pred.split(",")]

    # add artificialy pred at begin of y_list
    y_pred_list = [[i] + y_list for i in pred_list]

    # poiur chaque candidat pred -> evaluer la cosine similarity
    cos_y_pred_arrays = [sub_cosine_similarity(i).toarray() for i in y_pred_list]

    # prendre pour chaque pred le 1er ligne et oublier le 1er chiffre (cf matrice identité probkem)
    cos_y_pred_list = np.array([i[0][1:] for i in cos_y_pred_arrays])

    # prendre le max de chaque lignes
    max_cos_y_pred = np.array([max(i) for i in cos_y_pred_list])

    # soit retour de la liste restante
    # soit mean de cette list

    return max_cos_y_pred.mean()


def softmax(x):
    """Compute softmax values for each sets of scores in x."""

    e_x = np.exp(x - np.max(x))
    return e_x / e_x.sum(axis=0)  # only difference # correct solution:


def load_data(file_path: str) -> str:
    """from file_path open read and return text; return text """

    if ".pdf" in file_path:
        raise AttributeError("Error : file recieved is a pdf, only txt supported")

    with open(file_path, "r") as f:
        txt = f.read()

    return txt


def make_dataframe(path: str = "./data/csv/files.csv"):
    """ """

    df = pd.read_csv(path)
    return df


def uniquize(iterable: list) -> list:
    """ """

    try:
        return list(set(iterable))
    except Exception as e:
        return []


def strize(item_list, sep="\n", force_list=False):
    """ """

    # if score -1
    non_null = [(i, j) for i, j in item_list if j > -1]
    if not non_null:
        return ""

    # clean and unique
    clean_l = [str(i).replace("\n", "").strip() for i, j in non_null]
    str_cand = uniquize(clean_l)
    if not str_cand:
        return ""

    # return
    if len(str_cand) == 1:
        return str_cand[0]
    return sep.join(str_cand)


def get_spacy():
    """ """

    try:
        nlspa = spacy.load("en_core_web_sm")

    except Exception as e:
        call(["python", "-m", "spacy", "download", "en_core_web_sm"])

        nlspa = spacy.load("en_core_web_sm")
    try:
        nlspa.add_pipe("sentencizer")
        return nlspa
    except Exception as e:
        return nlspa


def _if_not_spacy(nlspa):
    """ if  not nlpipeline instance and return it else return pipeline already exists"""

    return nlspa if nlspa else get_spacy()


def get_label_(txt: str, label: str, nlspa=None) -> list:
    """check if a label in a text"""

    # print(label)
    nlspa = _if_not_spacy(nlspa)

    label = label.upper().strip()
    assert label in ["PERSON", "ORG", "MONEY", "DATE", "GPE"]

    ans = [i for i in nlspa(txt).ents if i.label_ == label]
    ans = [str(p) for p in ans]

    return ans


def get_pipeline():
    """ build and return a piplein"""

    return pipeline(
        "question-answering",
        model="distilbert-base-cased-distilled-squad",
        tokenizer="distilbert-base-cased",
    )


def _if_not_pipe(nlpipe):
    """ if  not nlpipeline instance and return it else return pipeline already exists"""

    return nlpipe if nlpipe else get_pipeline()


def _ask(txt: str, quest: str, nlpipe, topk: int = 3) -> list:
    """MAKE A QUESTION """

    # txt
    if not txt:
        raise AttributeError(f"Attribute error txt ; txt is {txt}, format {type(txt)}")

    # quest
    if not quest:
        raise AttributeError(
            f"Attribute error quest ; quest is {quest}, format {type(quest)}"
        )

    nlpipe = _if_not_pipe(nlpipe)

    return nlpipe(question=quest, context=txt, topk=topk)


def ask_all(txt, quest_pairs, sent_id=None, sent=None, nlpipe=None) -> list:
    """asl all questions and return a list of dict """

    # txt
    if not txt:
        raise AttributeError(f"Attribute error txt ; txt is {txt}, format {type(txt)}")

    # pipe
    nlpipe = _if_not_pipe(nlpipe)

    # ans
    ans = []

    # loop
    # logger.info(f"quest_pairs : {quest_pairs} , len {quest_pairs} ")
    # pdb.set_trace()

    for quest, label in quest_pairs:

        # logger.info(f"quest_pairs : {quest_pairs} ")
        ds = _ask(txt=txt, quest=quest, nlpipe=nlpipe)
        _ = [d.update({"question": quest, "quest_label": label}) for d in ds]
        if sent_id:
            _ = [d.update({"sent_id": sent_id}) for d in ds]
        if sent:
            _ = [d.update({"sent": sent}) for d in ds]

        ans.extend(ds)

    # sort
    ans = sorted(ans, key=lambda i: i["score"], reverse=True)

    return ans


def merge_ans(ans, label="new_answer", threshold=0.1):
    """based on new_answer we will make a groupby adding the scores for each new ans in a cumulative score
    example [{new_ans : hello, score:0.3},{new_ans : hello, score:0.3}, ]
    will become  [{new_ans : hello, score:0.6},]"""

    # build dataframe
    df = pd.DataFrame(ans)

    # check
    if not label in df.columns:
        raise AttributeError(
            f"pb  label in df.columns --> label is {label } cols are {df.columns}"
        )

    # select
    droped = [i for i in df.columns if i not in ["score", label]]
    df = df.drop(droped, axis=1, inplace=False)

    # group by ans and make cumutavie score of accuracy
    ll = [
        {label: k, "cum_score": round(v.score.sum(), 2)}
        for k, v in df.groupby(label)
        if v.score.sum() > threshold
    ]
    ll = sorted(ll, key=lambda i: i["cum_score"], reverse=True)

    return ll


def main_X_y(
    path: str = "./data/csv/", y: str = "random_y", text: str = "random_text"
) -> pd.DataFrame:
    """ """

    cands = os.listdir(path)
    text_file = [i for i in cands if text in i][0]
    y_file = [i for i in cands if y in i][0]

    text_df = pd.read_csv(path + text_file)
    y_df = pd.read_csv(path + y_file)

    drop_cols = [i for i in y_df.columns if "link" in i]
    y_df.drop(drop_cols, axis=1, inplace=True)
    y_df.drop("juridiction", axis=1, inplace=True)

    new_df = text_df.merge(y_df, on="folder", how="inner", copy=True)

    return new_df


class Utils:
    """ """

    # df
    main_X_y = main_X_y

    # predict
    ask = _ask
    ask_all = ask_all
    merge_ans = merge_ans

    # pipeline
    if_not_pipe = _if_not_pipe
    get_pipeline = get_pipeline

    # spacy
    if_not_spacy = _if_not_spacy
    get_spacy = get_spacy

    # label
    get_label = get_label_

    # version
    version = "2.2.1"



from legal_doc_processing import logger

from legal_doc_processing.utils import uniquize as _u
from legal_doc_processing.utils import merge_ans, ask_all

# from legal_doc_processing.legal_doc.clean.extracted_authorities import _filter_jur


def _question_helper(txt) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    k_list = [
        "reason",
        # "accused",
        # "defendant",
        "violate",
        "against",
        "filed",
        "judgement",
        "complaint",
    ]
    for kk in k_list:
        if kk in _txt:
            res.append(kk)

    return res


def _question_selector(key: str):
    """based on a key from _question helper find the list of good question to ask """

    qs = list()

    if "reason" in key:
        qs.extend(
            [
                ("Who has reason?", "who_reason"),
            ]
        )
    elif "filed" in key:
        qs.extend(
            [
                ("Who has filed?", "who_filed"),
                ("Who filed?", "who_filed"),
            ]
        )
    # else:
    #     qs.extend(
    #         [
    #             ("Who has reason?", "who_reason"),
    #             ("Who has filed?", "who_filed"),
    #         ]
    #     )

    return qs


def predict_extracted_authorities(obj: dict, threshold: int = 0.2, n_sents: int = 3):

    # pipe, spa
    nlspa = obj["nlspa"]

    # choose the item
    h1, abstract = obj["h1"], obj["abstract"]

    # token filter abstract
    tok_abstract = [i.text.lower() for i in nlspa(abstract)]
    jur_abstract = [_filter_jur(i) for i in tok_abstract]
    jur_abstract_clean = _u([i for i in jur_abstract if i])

    # juri abstract
    if len(jur_abstract_clean) >= 1:
        return [(i, 1) for i in jur_abstract_clean]

    return [(str(-3), -1)]


if __name__ == "__main__":

    # import
    import time
    from legal_doc_processing.utils import get_pipeline, get_spacy
    from legal_doc_processing.legal_doc.utils import legal_doc_X_y
    from legal_doc_processing.legal_doc.legal_doc import LegalDoc

    # laod
    nlpipe = get_pipeline()
    nlspa = get_spacy()
    nlspa.add_pipe("sentencizer")

    # data
    threshold = 0.2
    n_sents = 7
    feature = "extracted_authorities"

    # structured_press_release_r
    df = legal_doc_X_y(features="defendant")
    df = df.iloc[:2, :]
    df["ld"] = [LegalDoc(i, nlpipe=nlpipe, nlspa=nlspa) for i in df.txt.values]

    # preds
    t = time.time()
    # 28 objects --> 181 secondes so --> +/-10 secondes per objects
    df["pred_" + feature] = df.ld.apply(lambda i: i.predict(feature))
    t = time.time() - t

    # 1st one
    one = df.iloc[0, :]
    one_txt = one.txt
    one_ld = self = one.ld
    obj = self.data


import os
from pprint import pformat, pprint

from legal_doc_processing import logger

from legal_doc_processing.utils import get_label_

from legal_doc_processing.press_release.structure.utils import (
    clean_in_line_break,
    do_strip,
    clean_very_short_lines,
)


def _detect_true_text_id(txt_lines: str, line_length_txt=50, n_lines=4) -> int:
    """ """

    # find a first candidate for a REAL senetence
    # i, len char the line, the line itself
    cands_1st = [(i, len(j), j) for i, j in enumerate(txt_lines)]

    # then we want the len of this line, the len of the n+1 line, idem n+2, n+3
    cands_2nd = [
        (
            i,
            [cands_1st[i + k][1] for k in range(n_lines)],
            k,
        )
        for i, j, k in cands_1st[: -(n_lines + 1)]
    ]

    # we want to know if the 3 next lines will be len() > threshold
    cands_3rd = [(i, sum([kk > line_length_txt for kk in j]), k) for i, j, k in cands_2nd]

    # if not all then True
    cands_5th = [(i, j, k) for i, j, k in cands_3rd if j >= n_lines]

    if not len(cands_5th):
        return -1
    else:
        return cands_5th[0][0]


def split_intro_article_2(txt: str, n=30) -> str:
    """ """

    txt_other = txt.splitlines()[n:]
    txt_lines_30 = txt.splitlines()[:n]

    # find v.
    cand_list = ["SEC v.", "Commission v."]
    cond_ok = lambda i: any([cand.lower() in i.lower() for cand in cand_list])
    idx_v_cands = [i for i, j in enumerate(txt_lines_30) if cond_ok(j)]
    # logger.info(idx_v_cands)

    # cut v.
    idx_v_plus_1 = idx_v_cands[0] + 1
    pre_txt_lines = txt_lines_30[:idx_v_plus_1]
    post_txt_lines = txt_lines_30[idx_v_plus_1:]

    # true text begins at
    idx_true_txt = _detect_true_text_id(post_txt_lines)

    A_post_txt_lines = post_txt_lines[:idx_true_txt]
    B_post_txt_lines = post_txt_lines[idx_true_txt:]

    intro = "\n".join(pre_txt_lines + A_post_txt_lines)
    article = "\n".join(B_post_txt_lines + txt_other)

    return intro, article


def extract_id_2(intro: str) -> tuple:
    """ """

    # slplit
    # intro_lines = intro.splitlines()
    # logger.info(intro_lines)
    # intro_cleaned_lines = [i for i in intro_lines if i.strip()]
    # logger.info(intro_cleaned_lines)

    # # find v.
    # cand_list = ["SEC v.", "Commission v."]
    # cond_ok = lambda i: any([cand.lower() in i.lower() for cand in cand_list])
    # idx_v_cands = [i for i, j in enumerate(intro_lines) if cond_ok(j)]
    # logger.info(idx_v_cands)

    # idx_v = idx_v_cands[0] + 1
    # _id = intro_cleaned_lines[idx]

    # # clean intro
    # idx_cands = [i for i, j in enumerate(intro_lines) if _id in j]
    # idx = idx_cands[0]
    # intro_lines[idx] = ""
    # intro_ok = "\n".join(intro_lines)

    # return _id, intro_ok

    pass


def extract_date_2(intro: str, nlspa) -> tuple:
    """ """

    date_list = get_label_(intro, label="DATE", nlspa=nlspa)

    # if date_list
    if len(date_list) > 0:
        date = date_list[0]
    else:
        return "-1", intro

    # find date_line
    intro_lines = intro.splitlines()
    idx_list = [i for i, j in enumerate(intro_lines) if date.lower() in j.lower()]
    idx = idx_list[0]

    # clean intro
    intro_lines[idx] = ""
    intro_ok = "\n".join(intro_lines)

    return date, intro_ok


def extract_h1_2(intro: str) -> tuple:
    """ """

    intro_lines = intro.splitlines()

    # find v.
    cand_list = ["SEC v.", "Commission v."]
    cond_ok = lambda i: any([cand.lower() in i.lower() for cand in cand_list])
    idx_v_cands = [i for i, j in enumerate(intro_lines) if cond_ok(j)]
    # logger.info(idx_v_cands)

    # cut v.
    idx_v_plus_1 = idx_v_cands[0] + 1
    post_txt_lines = intro_lines[idx_v_plus_1:]

    # clean
    clean_list = [
        "CV-",
        "Case No.",
        "Release",
        "No.",
        "United States District",
        "(GEB)",
        "D.N.J.",
        "ET AL",
        "U.S.D.C.",
        "D.D.C.",
        "District of",
        "S.D.N.Y",
        ")",
    ]
    for clean_i in clean_list:
        post_txt_lines = [i for i in post_txt_lines if clean_i.lower() not in i.lower()]

    post_txt_lines = [i.strip() for i in post_txt_lines]

    if post_txt_lines[0] == "":
        post_txt_lines = post_txt_lines[1:]

    post_txt_lines = [(". " if i == "" else i) for i in post_txt_lines]

    h1 = "".join(post_txt_lines).strip()
    h1 = h1 if h1[-1] == "." else h1 + "."
    h1 = h1 if h1[-1] == "." else h1 + "."

    return h1, "-1"



from legal_doc_processing.utils import uniquize as _u


_key_list = [
    "require",
    "impose",
    "order",
    "pay",
    "forbid",
    "aggre",
    "penalt",
    "ban",
    "cease",
]


def _question_helper(txt: str) -> list:
    """txt"""

    _txt = txt.lower()
    res = list()

    res = [k for k in _key_list if k in _txt]

    return res


def _question_selector(key: str) -> list:
    """based on a key from _question helper find the list of good question to ask """

    qs = list()

    if "cease" in key:
        qs.extend(
            [
                ("What have they to do ?", "what_cease"),
            ]
        )

    if "ban" in key:
        qs.extend(
            [
                ("What are they banned from?", "what_ban"),
                ("What is the ban?? ", "what_ban"),
            ]
        )

    if "penalt" in key:
        qs.extend(
            [
                ("What is the penalty?", "what_penalt"),
                ("What are the penalties? ", "what_penalt"),
            ]
        )

    if "aggre" in key:
        qs.extend(
            [
                ("What have they agreed?", "what_agreement"),
                ("What is the agreement? ", "what_agreement"),
            ]
        )
    if "require" in key:  # accused
        qs.extend(
            [
                ("What is required?", "what_acusation"),
                ("What is required to do? ", "what_acusation"),
            ]
        )
    if "impose" in key:
        qs.extend(
            [
                ("What is imposed?", "what_acusation"),
                ("What is imposed to do?", "what_acusation"),
            ]
        )
    if "forbid" in key:
        qs.extend(
            [
                ("What is forbidden?", "what_acusation"),
            ]
        )

    if "order" in key:
        qs.extend(
            [
                ("What is ordered?", "what_acusation"),
            ]
        )

    if "pay" in key:
        qs.extend(
            [
                ("What they have to pay? ", "what_acusation"),
            ]
        )
    if not key:
        qs.extend(
            [
                ("what is the injunction?", ""),
                ("they are ordered to do what?", ""),
                ("what have they to do?", ""),
                ("What is the sentence?", "what_payement"),
                ("What is the punition?", "what_payement"),
                ("What have they to do?", "what_payement"),
                ("What are their obligations?", "what_payement"),
                ("What is the condemnation ?", ""),
                ("What is forbidden?", ""),
                ("What is imposed?", ""),
            ]
        )
    qs = _u(qs)

    return qs


def _question_lister(key_list: list) -> list:
    """from key_list return question list """

    question_list = []
    for key in key_list:
        question_list.extend(_question_selector(key))

    return _u(question_list)


import time
import random

import pandas as pd

from legal_doc_processing import press_release as pr
from legal_doc_processing import legal_doc as ld
from legal_doc_processing.utils import get_pipeline, get_spacy


# %time
nlpipe, nlspa = get_pipeline(), get_spacy()


test_press_txt = """
Release Number 7100-15

 

January 12, 2015

Federal Court in Florida Enters Order Freezing Assets in CFTC Foreign Currency Anti-
Fraud Action against Allied Markets LLC and its Principals Joshua Gilliland and Chawalit
Wongkhiao

CFIC Charges Jacksonville, Florida, Defendants with Operating a
Fraudulent Forex Pool and Misappropriating Customer Funds to Pay
Personal Expenses

Washington, DC — The U.S. Commodity Futures Trading Commission (CFTC) today announced that it filed a civil
enforcement Complaint in the U.S. District Court for the Middle District of Florida, charging Defendants Allied Markets
LLC, and its principals Joshua Gilliland and Chawalit Wongkhiao, all of Jacksonville, Florida, with operating a
fraudulent foreign currency (forex) commodity pool in violation of the Commodity Exchange Act (CEA) and CFTC
Regulations. In addition, none of the Defendants has ever been registered with the CFTC, as required.

"""

test_press_obj = pr.PressRelease(test_press_txt, nlpipe=nlpipe, nlspa=nlspa)
test_press_preds = test_press_obj.predict_all()


def decore_preds(obj, i=None, I=None) -> dict:
    """prevents errors  """

    if i and I:
        print(f"{i}/{I}", end=" ")

    t = time.time()
    try:
        print(str(obj.__repr__())[10:70])
        res = obj.predict_all()
        t = round(time.time() - t, 2)
        print((str(res)[:30]), end=" ")
        print(f"---> {t} secs <---\n")
        return res

    except Exception as e:
        t = round(time.time() - t, 2)
        print(e, end=" ")
        print(f"---> {t} secs <---\n")
        return {}


def compute_slices(len_df, batches_size_max=30):
    """ """

    len_df += 1
    batches_nb = len_df // batches_size_max
    slices = len_df // batches_nb
    extra = len_df % batches_nb

    # seps = [(0, 60), (60, 120),(120, 180), (180, 240) ]
    steps = [(i * slices, (i * slices) + slices) for i in range(batches_nb)]
    steps[-1] = (steps[-1][0], steps[-1][1] + extra)
    print(steps)

    return steps


def batch_pred_and_save(df_list, SOURCE, pred_name="__preds"):

    fn_list = list()
    rd = random.randint(1000, 9999)

    # for each sub df
    for i, sub_df in enumerate(df_list):

        print(f"---------\n{i}/{len(df_list)}\n---------\n\n")

        len_k = len(sub_df.press_obj)
        enn = enumerate(sub_df.press_obj)
        fn = f"{SOURCE}/data/csv/__{rd}__{SOURCE}_pred_df_{i}.csv"

        sub_df[pred_name] = [decore_preds(obj, k, len_k) for k, obj in enn]
        sub_df.to_csv(fn, index=False)

        fn_list.append(fn)

        time.sleep(3)

    return fn_list


def decore_dict(dd: dict, k: str) -> str:
    """ """

    try:
        return str(dd[k])
    except:
        return ""


def split_features(df, pred_name="__preds"):
    """ """

    one = df.iloc[0, :]
    keys = list(one[pred_name].keys())
    print(keys)

    for k in keys:
        df[k] = df[pred_name].apply(lambda dd: decore_dict(dd, k))

    return df



import os


def save_files(final_df, path="data/files/"):
    """ """

    for i, ser in final_df.iterrows():

        folder = ser.folder
        lenn = len(final_df)
        doc_filename = ser.document_link_new.split("/")[-1]
        press_filename = ser.press_release_link_new.split("/")[-1]
        doc_text = ser.legal_doc_text
        press_text = ser.press_release_text

        strize = lambda i: i[:30].replace("\n", "/n")

        try:
            print(f"i/I: {i}/{lenn}, folder : {folder}, path + folder : {path + folder}")
            print(f"doc_filename : {doc_filename}, press_filename : {press_filename}")
            print(f"doc_text : {strize(doc_text)} ||| press_text : {strize(press_text)}")
            print()

            # mkdir
            if not os.path.isdir(path + folder):
                os.mkdir(path + folder)

            # doc.txt and press .txt
            for fn, txt in [(doc_filename, doc_text), (press_filename, press_text)]:
                with open(path + folder + "/" + fn, "w") as f:
                    f.write(txt)

        except Exception as e:
            pass
